/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * DynamicGridMap.h
 *
 *  Created on: Jul 12, 2010
 *      Author: lziegler, jwienke
 */

#ifndef DYNAMICGRIDMAP_H_
#define DYNAMICGRIDMAP_H_

#include "../../Type.h"

#include <vector>

#include <xmltio/xmltio.hpp>

#include <boost/shared_ptr.hpp>

namespace airobots {

/**
 * This local class is a container for the binary data of DynamicGridMap, that
 * enables easy access to the bins of the grid map.
 */
class GridMap {
public:

	GridMap() :
		nrows_(0), ncols_(0) {
		data_ = new float[1];
	}
	GridMap(unsigned nrows, unsigned ncols) :
		nrows_(nrows), ncols_(ncols) {
		if (nrows == 0 || ncols == 0)
			throw BadSize();
		data_ = new float[nrows * ncols];
	}
	// Throws a BadSize object if either size is zero
	class BadSize {
	};

	~GridMap() {
		delete[] data_;
	}
	GridMap(GridMap const& m) :
		nrows_(m.nrows_), ncols_(m.ncols_) {
		data_ = new float[nrows_ * ncols_];
		memcpy(data_, m.data_, nrows_ * ncols_ * sizeof(float));
	}
	GridMap& operator=(GridMap const& m) {

		if (nrows_ != 0 && ncols_ != 0) {
			delete[] data_;
		}

		nrows_ = m.nrows_;
		ncols_ = m.ncols_;
		data_ = new float[nrows_ * ncols_];
		memcpy(data_, m.data_, nrows_ * ncols_ * sizeof(float));
		return *this;
	}

	// Access methods to get the (i,j) element:
	inline float& operator()(unsigned row, unsigned col) {
		if (row >= nrows_ || col >= ncols_)
			throw BoundsViolation();
		return data_[row * ncols_ + col];
	}
	inline float const& operator()(unsigned row, unsigned col) const {
		if (row >= nrows_ || col >= ncols_)
			throw BoundsViolation();
		return data_[row * ncols_ + col];
	}
	// These throw a BoundsViolation object if i or j is too big
	class BoundsViolation {
	};

private:
	unsigned nrows_, ncols_;
	float* data_;
};

/**
 * This is a container for dynamically resizing grid map data like used in the
 * slam approach. This includes a grid map representing the binary data and
 * meta information like resolution and origin of the map.
 *
 * @author lziegler
 */
class DynamicGridMap: public Type {

public:

	static std::string getBaseTag();

	static std::string getTemplate();

	DynamicGridMap();
	DynamicGridMap(std::string generator);
	DynamicGridMap(const DynamicGridMap &map);
	DynamicGridMap& operator=(DynamicGridMap const& map);
	virtual ~DynamicGridMap();

	/**
	 * Returns the attachment uri of the binary slam map.
	 */
	std::string getUri() const;
	void setUri(std::string uri);

	/**
	 * Returns the x size of the map in map units.
	 */
	unsigned int getSizeX() const;
	void setSizeX(unsigned int sizeX);

	/**
	 * Returns the y size of the map in map units.
	 */
	unsigned int getSizeY() const;
	void setSizeY(unsigned int sizeY);

	/**
	 * Returns the x origin of the map in map units.
	 */
	unsigned int getOriginX() const;
	void setOriginX(unsigned int originX);

	/**
	 * Returns the y origin of the map in map units.
	 */
	unsigned int getOriginY() const;
	void setOriginY(unsigned int originY);

	/**
	 * Returns the resolution of the map. That is:
	 * map units * resolution = meters.
	 */
	float getResolution() const;
	void setResolution(float resolution);

	/**
	 * Returns the map itself.
	 */
	const GridMap& getGridMap() const;
	void setGridMap(GridMap &gridMap);
	void setGridMap(const std::vector<unsigned char> &charArray,
			unsigned int width, unsigned int height);
	void setGridMap(const std::vector<float> &floatArray, unsigned int width,
			unsigned int heigth);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, DynamicGridMap &type);

private:
	std::string uri;
	unsigned int sizeX;
	unsigned int sizeY;
	unsigned int originX;
	unsigned int originY;
	float resolution;
	GridMap gridMap;
};

/**
 * Smart pointer for DynamicGridMap
 */
typedef boost::shared_ptr<DynamicGridMap> DynamicGridMapPtr;

}

#endif /* BINARYSLAMMAP_H_ */
