package de.unibi.airobots.btl;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.ListIterator;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.Node;
import nu.xom.Nodes;
import org.apache.log4j.Logger;

/**
 * Generic type for lists of BTL types. Note that since the actual type of the
 * list elements is not known in a static context, calling the static methods
 * getBaseTag and fromElement could yield unexpected results. There is an
 * alternative getBaseTag(Class<T>) method that can be called with the element
 * type class and returns the base tag of the element type with the suffix LIST
 * appended to it.
 * 
 * @param <T>
 *            The type of the list elements.
 */
public class List<T extends Type> extends Type implements java.util.List<T> {

	private static final String BASE_TAG = "LIST";
	private static final String ELEMENT_TYPE_TAG = "ELEMENT_TYPE";
	public Class<T> elementType;
	public java.util.List<T> elements = new ArrayList<T>();
	private static Logger logger = Logger.getLogger(Type.class);

	@SuppressWarnings("unchecked")
	public static <T extends Type> Class<List<T>> getListClass(
			Class<T> type) {
		return (Class<List<T>>) new List<T>(type).getClass();
	}

	/**
	 * Default constructor as expected by {@link Type}. You should not use it!
	 * If you must use it call {@link #init(java.lang.Class)} afterwards.
	 * 
	 * @deprecated Use {@link List#List(java.lang.Class)} instead! This
	 *             constructor is only used by {@link Type}.
	 */
	@Deprecated
	public List() {
	}

	/**
	 * Create a new list.
	 * 
	 * @param type
	 *            The type of the elements.
	 */
	public List(Class<T> type) {
		init(type);
	}

	/**
	 * Initialize this list.
	 * 
	 * @param type
	 *            The type of the elements.
	 */
	public final void init(Class<T> type) {
		elements = new ArrayList<T>();
		elementType = type;
	}

	/**
	 * Getter for the type of the elements.
	 * 
	 * @return The type of the elements.
	 */
	public Class<T> getElementType() {
		return elementType;
	}

	@Override
	public boolean add(T t) {
		return elements.add(t);
	}

	@Override
	public T get(int index) {
		return elements.get(index);
	}

	@Override
	public boolean contains(Object o) {
		return elements.contains(o);
	}

	@Override
	public int size() {
		return elements.size();
	}

	/**
	 * Returns the tag name of the the root tag this type is usually used with
	 * in xml.
	 * 
	 * @return tag name
	 */
	public static String getBaseTag() {
		return BASE_TAG;
	}

	/**
	 * Returns the tag name of the the element tags this type is usually used
	 * with in xml.
	 * 
	 * @return tag name
	 * @param <T>
	 *            type of the list's elements
	 * @param type
	 *            Class object of the list element's type
	 */
	public static <T extends Type> String getBaseTag(Class<T> type) {
		String elementBaseTag = "";

		try {
			elementBaseTag = (String) type.getMethod("getBaseTag").invoke(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}

		return elementBaseTag + "LIST";
	}

	@Override
	public void sanitizeElement(Element parent) {
		super.sanitizeElement(parent);

		Elements typeElements = parent.getChildElements(ELEMENT_TYPE_TAG);
		for (int i = 0; i < typeElements.size(); i++) {
			parent.removeChild(typeElements.get(i));
		}

		try {
			String baseTag = (String) elementType.getMethod("getBaseTag")
					.invoke(null);
			Elements elemElements = parent.getChildElements(baseTag);
			for (int i = 0; i < elemElements.size(); i++) {
				parent.removeChild(elemElements.get(i));
			}
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error.");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (SecurityException e) {
			logger.fatal("Reflection error.");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error.");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error.");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error.");
			logger.debug(e);
			assert false : "Reflection error.";
		}
	}

	@Override
	public void fillInto(Element parent) {

		sanitizeElement(parent);
		super.fillInto(parent);

		// add elements' type to xml document
		Element elTypeElement = new Element(ELEMENT_TYPE_TAG);
		elTypeElement.appendChild(elementType.getName());
		parent.appendChild(elTypeElement);

		// fill in elements
		for (T p : elements) {
			try {
				String baseTag = (String) elementType.getMethod("getBaseTag")
						.invoke(null);
				Element perceptElement = new Element(baseTag);
				p.fillInto(perceptElement);
				parent.appendChild(perceptElement);

			} catch (IllegalArgumentException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (SecurityException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (IllegalAccessException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (InvocationTargetException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (NoSuchMethodException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			}
		}
	}

	/**
	 * Parses the contents of the given nu.xom.Element into the type instance.
	 * Therefore element must already be parsed into the base tag of the list
	 * type.
	 * 
	 * @param element
	 *            element to get the contents of this type from
	 * @param list
	 *            List to fill with the contents
	 * @param <T>
	 *            type of the list's elements.
	 */
	@SuppressWarnings("unchecked")
	public static <T extends Type> void fromElement(Element element,
			List<T> list) {
		Type.fromElement(element, list);
		Nodes nodes = null;
		try {

			Class<T> elementType;

			if (list.elementType == null) {

				// extract the elements' type from the element type tag
				Element elementTypeElement = element
						.getFirstChildElement(ELEMENT_TYPE_TAG);

				if (list.elementType == null && elementTypeElement == null) {
					logger.fatal("Parsing error: List types must have a "
							+ ELEMENT_TYPE_TAG + " tag!");
					assert false : "Parsing error: List types must have a "
							+ ELEMENT_TYPE_TAG + " tag!";
					return;
				}

				String elementTypeStr = elementTypeElement.getValue();
				elementType = (Class<T>) Class.forName(elementTypeStr);

				list.init(elementType);

			}

			// get the base tag for the elements
			String baseTag = (String) list.elementType.getMethod("getBaseTag")
					.invoke(null);
			nodes = element.query(baseTag);

		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error. Types must have a default "
					+ "constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (SecurityException e) {
			logger.fatal("Reflection error. SecurityException");
			logger.debug(e);
			assert false : "Reflection error. SecurityException";
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error. NoSuchMethodException");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error. IllegalArgumentException");
			logger.debug(e);
			assert false : "Reflection error. IllegalArgumentException";
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error. InvocationTargetException. "
					+ list.elementType + " is missing a getBaseTag method");
			logger.debug(e);
			assert false : "Reflection error. InvocationTargetException";
		} catch (ClassNotFoundException e) {
			logger.fatal("Reflection error. ClassNotFoundException. Tag "
					+ ELEMENT_TYPE_TAG
					+ " must contain a valid class name describing the "
					+ "element's types.");
			logger.debug(e);
			assert false : "Reflection error. ClassNotFoundException";
		}

		try {

			// parse elements
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);

				if (node instanceof Element) {
					Element perceptElement = (Element) node;

					T parsed = list.elementType.newInstance();
					Method fromElement = list.elementType.getMethod(
							"fromElement", new Class[] { Element.class,
									list.elementType });
					fromElement.invoke(null, perceptElement, parsed);

					list.add(parsed);
				}
			}

		} catch (InstantiationException e) {
			logger.fatal("Reflection error. Types must have a default "
					+ "constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error. Types must have a default "
					+ "constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (SecurityException e) {
			logger.fatal("Reflection error. SecurityException");
			logger.debug(e);
			assert false : "Reflection error. SecurityException";
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error. NoSuchMethodException");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error. IllegalArgumentException");
			logger.debug(e);
			assert false : "Reflection error. IllegalArgumentException";
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error. InvocationTargetException. "
					+ list.elementType
					+ " is missing a fromElement method with "
					+ "correct signature or method threw exception");
			if (logger.isDebugEnabled()) {
				logger.debug(element.getDocument().toXML());
				e.printStackTrace();
			}
			
			assert false : "Reflection error. InvocationTargetException";
		}
	}

	@Override
	public void add(int index, T element) {
		elements.add(index, element);
	}

	@Override
	public boolean addAll(Collection<? extends T> c) {
		return elements.addAll(c);
	}

	@Override
	public boolean addAll(int index, Collection<? extends T> c) {
		return elements.addAll(index, c);
	}

	@Override
	public void clear() {
		elements.clear();
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		return elements.containsAll(c);
	}

	@Override
	public int indexOf(Object o) {
		return indexOf(o);
	}

	@Override
	public boolean isEmpty() {
		return elements.isEmpty();
	}

	@Override
	public Iterator<T> iterator() {
		return elements.iterator();
	}

	@Override
	public int lastIndexOf(Object o) {
		return elements.lastIndexOf(o);
	}

	@Override
	public ListIterator<T> listIterator() {
		return elements.listIterator();
	}

	@Override
	public ListIterator<T> listIterator(int index) {
		return elements.listIterator(index);
	}

	@Override
	public boolean remove(Object o) {
		return elements.remove(o);
	}

	@Override
	public T remove(int index) {
		return elements.remove(index);
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		return elements.removeAll(c);
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		return elements.retainAll(c);
	}

	@Override
	public T set(int index, T element) {
		return elements.set(index, element);
	}

	@Override
	public java.util.List<T> subList(int fromIndex, int toIndex) {
		return elements.subList(fromIndex, toIndex);
	}

	@Override
	public Object[] toArray() {
		return elements.toArray();
	}

	@Override
	public <T2> T2[] toArray(T2[] a) {
		return elements.toArray(a);
	}
}
