package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This class represents a point in space by all 3 dimensions.
 * 
 * @author lziegler
 */
public class Point3D extends Type {

	/**
	 * Scope of the coordinates.
	 * 
	 * @author lziegler
	 */
	public enum Scope {
		GLOBAL, LOCAL
	};

	// coordinates in millimeters
	private double x;
	private double y;
	private double z;
	private Scope scope = Scope.GLOBAL;

	/**
	 * Creates instance.
	 */
	public Point3D() {
	}

	public Point3D(Point3D p) {
		this.x = p.x;
		this.y = p.y;
		this.z = p.z;
		this.scope = p.scope;
	}

	
	/**
	 * Creates instance.
	 * 
	 * @param x
	 *            first value
	 * @param y
	 *            second value
	 * @param z
	 *            third value
	 * @param unit
	 *            unit of the values
	 */
	public Point3D(double x, double y, double z, LengthUnit unit) {
		setX(x, unit);
		setY(y, unit);
		setZ(z, unit);
	}

	/**
	 * Creates instance.
	 * 
	 * @param x
	 *            first value
	 * @param y
	 *            second value
	 * @param z
	 *            third value
	 * @param unit
	 *            unit of the values
	 * @param scope
	 *            the scope of the coordinates.
	 */
	public Point3D(double x, double y, double z, LengthUnit unit, Scope scope) {
		setX(x, unit);
		setY(y, unit);
		setZ(z, unit);
		this.scope = scope;
	}

	public double getX(LengthUnit unit) {
		return UnitConverter.convert(x, LengthUnit.MILLIMETER, unit);
	}

	public void setX(double x, LengthUnit unit) {
		this.x = UnitConverter.convert(x, unit, LengthUnit.MILLIMETER);
	}

	public double getY(LengthUnit unit) {
		return UnitConverter.convert(y, LengthUnit.MILLIMETER, unit);
	}

	public void setY(double y, LengthUnit unit) {
		this.y = UnitConverter.convert(y, unit, LengthUnit.MILLIMETER);
	}

	public double getZ(LengthUnit unit) {
		return UnitConverter.convert(z, LengthUnit.MILLIMETER, unit);
	}

	public void setZ(double z, LengthUnit unit) {
		this.z = UnitConverter.convert(z, unit, LengthUnit.MILLIMETER);
	}

	public Scope getScope() {
		return scope;
	}

	public void setScope(Scope scope) {
		this.scope = scope;
	}

	public static String getBaseTag() {
		return "POINT3D";
	}

	/**
	 * Constructs a {@link Point3D} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link Point3D} object containing all the information given
	 *         by the {@link Element} object.
	 */
	public static void fromElement(Element objectElement, Point3D object)
			throws ParsingException {

		try {

			double x0 = Double.parseDouble(objectElement.getAttribute("x")
					.getValue());
			double y0 = Double.parseDouble(objectElement.getAttribute("y")
					.getValue());
			double z0 = Double.parseDouble(objectElement.getAttribute("z")
					.getValue());

			Scope s0;
			try {
				s0 = Scope.valueOf(objectElement.getAttribute("scope")
						.getValue());
			} catch (IllegalArgumentException e) {
				s0 = Scope.GLOBAL;
			} catch (NullPointerException e) {
				s0 = Scope.GLOBAL;
			}

			object.setScope(s0);
			object.setX(x0, LengthUnit.MILLIMETER);
			object.setY(y0, LengthUnit.MILLIMETER);
			object.setZ(z0, LengthUnit.MILLIMETER);

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present

			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new ParsingException("could not parse coordinate values.", e);
		}
	}

	/**
	 * Constructs a {@link Point3D} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link Point3D} object containing all the information given
	 *         by the {@link Element} object.
	 */
	public static Point3D fromElement(Element objectElement) 
	throws ParsingException {
		Point3D p = new Point3D();
		fromElement(objectElement, p);
		return p;
	}

	/**
	 * Serializes the {@link Point3D} object into a given XOM {@link Element}.
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {

		parent.addAttribute(new Attribute("x", String
				.valueOf(getX(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("y", String
				.valueOf(getY(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("z", String
				.valueOf(getZ(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("scope", String.valueOf(scope)));
	}

	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof Point3D))
				return false;

			Point3D other = (Point3D) obj;

			if (other.getX(LengthUnit.MILLIMETER) != getX(LengthUnit.MILLIMETER))
				return false;
			if (other.getY(LengthUnit.MILLIMETER) != getY(LengthUnit.MILLIMETER))
				return false;
			if (other.getZ(LengthUnit.MILLIMETER) != getZ(LengthUnit.MILLIMETER))
				return false;
			if (other.getScope() != scope)
				return false;

		} catch (Exception e) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "[" + getBaseTag() + " x=" + x + " y=" + y + " z=" + z
				+ " scope=" + scope + "]";
	}

	public double distance(Point3D center) {
		return Math.sqrt(
				Math.pow(this.x-center.x, 2)+
				Math.pow(this.y-center.y, 2)+
				Math.pow(this.z-center.z, 2)
				);
	}
}
