package de.unibi.airobots.btl.data.geometry;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This is a serializable and iterable polygon type. It is similar to
 * {@link Polygon}, but has a double precision.
 * 
 * TODO: implement utility functions like those in {@link java.awt.Polygon}.
 * 
 * @author lziegler
 */
public class PrecisePolygon implements Iterable<Point2D> {

	/**
	 * List of polygon points.
	 */
	protected List<Point2D> list = new LinkedList<Point2D>();

	public PrecisePolygon(PrecisePolygon poly) {
		LengthUnit unit = LengthUnit.MILLIMETER;
		for (Point2D point : poly) {
			list.add(new Point2D(point.getX(unit), point.getY(unit), unit));
		}
	}

	public PrecisePolygon() {
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "PRECISEPOLYGON";
	}

	/**
	 * Constructs a {@link PrecisePolygon} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link PrecisePolygon} object containing all the information
	 *         given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static PrecisePolygon fromElement(Element objectElement)
			throws ParsingException {

		PrecisePolygon poly = new PrecisePolygon();

		try {

			if (!objectElement.getLocalName().equals(getBaseTag())) {
				throw new IllegalArgumentException("Parsing Error: Base tag "
						+ "for polygon object must be '" + getBaseTag() + "'");
			}

			Elements points = objectElement.getChildElements(Point2D
					.getBaseTag());
			for (int i = 0; i < points.size(); i++) {
				Element point = points.get(i);
				Point2D p = new Point2D();
				Point2D.fromElement(point, p);
				poly.addPoint(p);
			}

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(
					"could not parse coordinate values");
		}
		return poly;
	}

	public void addPoint(double x, double y, LengthUnit unit) {
		list.add(new Point2D(x, y, unit));
	}

	/**
	 * Adds a {@link Point2D} to the polygon.
	 * 
	 * @param p
	 *            The {@link Point2D} instance to add.
	 */
	public void addPoint(Point2D point) {
		list.add(point);
	}

	/**
	 * Serializes the {@link PrecisePolygon} object into a given XOM
	 * {@link Element}.
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {
		for (Point2D p : list) {
			Element item = new Element(Point2D.getBaseTag());
			p.fillInto(item);
			parent.appendChild(item);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String info = "[PRECISEPOLYGON";
		for (Point2D p : list) {
			info += " " + p.toString();
		}
		info += "]";
		return info;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof PrecisePolygon))
				return false;

			PrecisePolygon other = (PrecisePolygon) obj;

			return other.list.equals(list);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Returns the centroid of this polygon. Implemented as arithmetic mean
	 * value of all polygon points.
	 * 
	 * @return The centroid of this polygon.
	 */
	public Point2D getCentroid() {

		double x = 0;
		double y = 0;
		int count = 0;
		for (Point2D p : list) {
			x += p.getX(LengthUnit.MILLIMETER);
			y += p.getY(LengthUnit.MILLIMETER);
			count++;
		}

		if (count == 0) {
			return null;
		} else {
			return new Point2D(x / (double) count, y / (double) count,
					LengthUnit.MILLIMETER);
		}
	}

	public double getMinX(LengthUnit unit) {
		double minX = Double.POSITIVE_INFINITY;
		for (Point2D point : list) {
			double x = point.getX(unit);
			if (minX > x) {
				minX = x;
			}
		}
		return minX;
	}

	public double getMaxX(LengthUnit unit) {
		double maxX = Double.NEGATIVE_INFINITY;
		for (Point2D point : list) {
			double x = point.getX(unit);
			if (maxX < x) {
				maxX = x;
			}
		}
		return maxX;
	}

	public double getMinY(LengthUnit unit) {
		double minY = Double.POSITIVE_INFINITY;
		for (Point2D point : list) {
			double y = point.getY(unit);
			if (minY > y) {
				minY = y;
			}
		}
		return minY;
	}

	public double getMaxY(LengthUnit unit) {
		double maxY = Double.NEGATIVE_INFINITY;
		for (Point2D point : list) {
			double y = point.getY(unit);
			if (maxY < y) {
				maxY = y;
			}
		}
		return maxY;
	}

	public void move(double dx, double dy, LengthUnit unit) {
		for (Point2D point : list) {
			point.setX(point.getX(unit) + dx, unit);
			point.setY(point.getY(unit) + dy, unit);
		}
	}

	public void scale(double factor) {
		for (Point2D point : list) {
			point.setX(point.getX(LengthUnit.MILLIMETER) * factor,
					LengthUnit.MILLIMETER);
			point.setY(point.getY(LengthUnit.MILLIMETER) * factor,
					LengthUnit.MILLIMETER);
		}
	}

	/**
	 * Rotates this polygon around a given center coordinate by a given angle.
	 * 
	 * @param center
	 *            The center of the rotation.
	 * @param angle
	 *            The angle to rotate.
	 * @param unit
	 *            The unit of the given angle value.
	 */
	public void rotate(Point2D center, double angle, AngleUnit unit) {

		// ensure radiant unit
		angle = UnitConverter.convert(angle, unit, AngleUnit.RADIAN);

		// generate path
		GeneralPath path = generatePath(LengthUnit.MILLIMETER);

		// create an affine transformation representing the rotation
		AffineTransform trans = AffineTransform.getRotateInstance(angle,
				center.getX(LengthUnit.MILLIMETER),
				center.getY(LengthUnit.MILLIMETER));

		// apply transform
		path.transform(trans);

		// update path
		updateFromPath(path, LengthUnit.MILLIMETER);
	}

	/**
	 * Rotates this polygon around a given center coordinate by a given angle.
	 * 
	 * @param x
	 *            The center of the rotation.
	 * @param y
	 *            The center of the rotation.
	 * @param lengthUnit
	 *            Unit of the given center values.
	 * @param angle
	 *            The angle to rotate.
	 * @param angleUnit
	 *            The unit of the given angle value.
	 */
	public void rotate(double x, double y, LengthUnit lengthUnit, double angle,
			AngleUnit angleUnit) {

		// ensure radiant unit
		angle = UnitConverter.convert(angle, angleUnit, AngleUnit.RADIAN);

		// generate path
		GeneralPath path = generatePath(lengthUnit);

		// create an affine transformation representing the rotation
		AffineTransform trans = AffineTransform.getRotateInstance(angle, x, y);

		// apply transform
		path.transform(trans);

		// update path
		updateFromPath(path, lengthUnit);
	}

	/**
	 * Tests if the given coordinate lies inside this polygon.
	 * 
	 * @param x
	 *            X-coordinate of the point to check.
	 * @param y
	 *            Y-coordinate of the point to check.
	 * @param unit
	 *            The unit of the given values.
	 * @return <code>true</code> if the given coordinate lied inside this
	 *         polygon.
	 */
	public boolean contains(double x, double y, LengthUnit unit) {
		if (list.size() < 3)
			return false;
		return generatePath(unit).contains(x, y);
	}

	/**
	 * Tests if the given coordinate lies inside this polygon.
	 * 
	 * @param p
	 *            Coordinate to check.
	 * @return <code>true</code> if the given coordinate lied inside this
	 *         polygon.
	 */
	public boolean contains(Point2D p) {
		return contains(p.getX(LengthUnit.METER), p.getY(LengthUnit.METER),
				LengthUnit.METER);
	}

	/**
	 * Calculates the area of this polygon.
	 * 
	 * @param unit
	 *            The desired length unit of the resulting value.
	 * @return The area of this polygon in the desired unit.
	 */
	public double getArea(LengthUnit unit) {

		double minX = getMinX(unit);
		double minY = getMinY(unit);

		// found here: http://www.exaflop.org/docs/cgafaq/cga2.html
		double sum = 0;
		for (int i = 0; i < list.size() - 1; i++) {

			// no negative values
			double x = list.get(i).getX(unit) + minX;
			double xp1 = list.get(i + 1).getX(unit) + minX;
			double y = list.get(i).getY(unit) + minY;
			double yp1 = list.get(i + 1).getY(unit) + minY;

			sum = sum + x * yp1 - y * xp1;
		}
		return Math.abs(sum / 2.0);
	}

	private GeneralPath generatePath(LengthUnit unit) {
		GeneralPath path = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
		path.reset();
		path.moveTo(list.get(0).getX(unit), list.get(0).getY(unit));
		for (Point2D point : list) {
			path.lineTo(point.getX(unit), point.getY(unit));
		}
		path.closePath();
		return path;
	}

	private void updateFromPath(GeneralPath path, LengthUnit unit) {
		double[] point = new double[6];
		PathIterator iterator = path.getPathIterator(null);
		int count = 0;
		while (!iterator.isDone()) {
			int type = iterator.currentSegment(point);
			if (type == PathIterator.SEG_LINETO) {
				list.get(count).setX(point[0], unit);
				list.get(count).setY(point[1], unit);
				count++;
			}
			iterator.next();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Iterator<Point2D> iterator() {
		return list.iterator();
	}
}
