package de.unibi.airobots.btl.data.map;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;

public class SlamUpdateData extends Type {

	/**
	 * Grid cells in metric coordinates.
	 * 
	 * @author jwienke
	 * @author mhanheid
	 */
	public class SlamUpdateGrid {

		public final double TOLERANCE = 0.00001;

		public SlamUpdateGrid() {
		}

		public SlamUpdateGrid(double x, double y) {
			this.x = x;
			this.y = y;
		}

		public double x;
		public double y;

		@Override
		public boolean equals(Object obj) {

			if (!(obj instanceof SlamUpdateGrid)) {
				return false;
			}

			SlamUpdateGrid other = (SlamUpdateGrid) obj;

			return Math.abs(other.x - x) < TOLERANCE
					&& Math.abs(other.y - y) < TOLERANCE;

		}

	}

	private final static Logger logger = Logger.getLogger(SlamUpdateData.class);
	private List<SlamUpdateGrid> additions;
	private List<SlamUpdateGrid> removals;
	private String type;

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	/**
	 * A SlamUpdateMap holds a SlamPos and a SlamMap and the update elements
	 * 
	 */
	public SlamUpdateData() {
		this.additions = new LinkedList<SlamUpdateGrid>();
		this.removals = new LinkedList<SlamUpdateGrid>();
		this.clear();
	}

	public List<SlamUpdateGrid> getAdditions() {
		return additions;
	}

	public void setAdditions(List<SlamUpdateGrid> additions) {
		this.additions = additions;
	}

	public void addAddition(double x, double y) {
		logger.debug("adding addition (" + x + ", " + y + ")");
		SlamUpdateGrid coords = new SlamUpdateGrid();
		coords.x = x;
		coords.y = y;
		additions.add(coords);
	}

	public void addRemoval(double x, double y) {
		logger.debug("adding removal (" + x + ", " + y + ")");
		SlamUpdateGrid coords = new SlamUpdateGrid();
		coords.x = x;
		coords.y = y;
		removals.add(coords);
	}

	public void clear() {
		logger.debug("clear");
		additions.clear();
		removals.clear();
	}

	public List<SlamUpdateGrid> getRemovals() {
		return removals;
	}

	public void setRemovals(List<SlamUpdateGrid> removals) {
		this.removals = removals;
	}

	public static String getBaseTag() {
		return "SLAMUPDATE";
	}

	public static void fromElement(Element element, SlamUpdateData type)
			throws ParsingException {

		Type.fromElement(element, type);

		type.setType(element.getAttributeValue("type"));

		Nodes additionNodes = element.query("ADDITIONS/GRID");
		for (int i = 0; i < additionNodes.size(); i++) {
			if (additionNodes.get(i) instanceof Element) {
				Element gridElement = (Element) additionNodes.get(i);
				type.addAddition(ElementParser.getDoubleAttributeValue(
						gridElement, "metric_x"), ElementParser
						.getDoubleAttributeValue(gridElement, "metric_y"));
			}
		}

		Nodes removalNodes = element.query("REMOVALS/GRID");
		for (int i = 0; i < removalNodes.size(); i++) {
			if (removalNodes.get(i) instanceof Element) {
				Element gridElement = (Element) removalNodes.get(i);
				type.addRemoval(ElementParser.getDoubleAttributeValue(
						gridElement, "metric_x"), ElementParser
						.getDoubleAttributeValue(gridElement, "metric_y"));
			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		logger.debug("fillInto");

		Attribute attType = new Attribute("type", type);
		parent.addAttribute(attType);
		logger.debug("root = " + parent);

		// additions
		Element additionsElem = new Element("ADDITIONS");
		Iterator<SlamUpdateGrid> it = additions.iterator();
		while (it.hasNext()) {
			SlamUpdateGrid elem = it.next();
			Element grid = new Element("GRID");
			Attribute attmx = new Attribute("metric_x", Double.toString(elem.x));
			Attribute attmy = new Attribute("metric_y", Double.toString(elem.y));
			grid.addAttribute(attmx);
			grid.addAttribute(attmy);
			additionsElem.appendChild(grid);
		}
		logger.debug("generated ADDITIONS element: " + additionsElem.toXML());
		parent.appendChild(additionsElem);

		// removals
		Element removalsElem = new Element("REMOVALS");
		it = removals.iterator();
		while (it.hasNext()) {
			SlamUpdateGrid elem = it.next();
			Element grid = new Element("GRID");
			Attribute attmx = new Attribute("metric_x", Double.toString(elem.x));
			Attribute attmy = new Attribute("metric_y", Double.toString(elem.y));
			grid.addAttribute(attmx);
			grid.addAttribute(attmy);
			removalsElem.appendChild(grid);
		}
		parent.appendChild(removalsElem);

	}
}
