package de.unibi.airobots.btl.data.navigation;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.airobots.btl.Type;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import org.apache.log4j.Logger;

/**
 * This class represents a vector consisting of configuration values. The values
 * set to each parameter of the configuration allow for manipulation of the
 * robots path planning. Receivers of this type need to be able to forward it to
 * a ROS dynamic_reconfiguration server (e.g.
 * /move_base/local_costmap/set_parameters) as is ros4xcf. An explanation on the
 * wide range of parameters can be found in section 3.2 here:
 * http://www.ros.org/wiki/costmap_2d
 *
 * The purpose of sending this type is to change certain parameters of a
 * costmap_2d node while it is already running, allowing for on the fly robot
 * navigation behavior change/adaption.
 *
 * @author pdressel
 */
public class ROSCostmapConfiguration extends ROSDynamicConfiguration {

    private static final String COSTMAP_CONFIGURATION_TAG = "COSTMAPCONFIGURATION";
    
    public static final String TRANSFORM_TOLERANCE_ATTRIBUTE = "transform_tolerance";
    public static final String UPDATE_FREQUENCY_ATTRIBUTE = "update_frequency";
    public static final String PUBLISH_FREQUENCY_ATTRIBUTE = "publish_frequency";
    public static final String MAX_OBSTACLE_HEIGHT_ATTRIBUTE = "max_obstacle_height";
    public static final String MAX_OBSTACLE_RANGE_ATTRIBUTE = "max_obstacle_range";
    public static final String RAYTRACE_RANGE_ATTRIBUTE = "raytrace_range";
    public static final String COST_SCALING_FACTOR_ATTRIBUTE = "cost_scaling_factor";
    public static final String INFLATION_RADIUS_ATTRIBUTE = "inflation_radius";
    public static final String FOOTPRINT_ATTRIBUTE = "footprint";
    public static final String ROBOT_RADIUS_ATTRIBUTE = "robot_radius";
    public static final String STATIC_MAP_ATTRIBUTE = "static_map";
    public static final String ROLLING_WINDOW_ATTRIBUTE = "rolling_window";
    public static final String UNKNOWN_COST_VALUE_ATTRIBUTE = "unknown_cost_value";
    public static final String WIDTH_ATTRIBUTE = "width";
    public static final String HEIGHT_ATTRIBUTE = "height";
    public static final String RESOLUTION_ATTRIBUTE = "resolution";
    public static final String ORIGIN_X_ATTRIBUTE = "origin_x";
    public static final String ORIGIN_Y_ATTRIBUTE = "origin_y";
    public static final String PUBLISH_VOXEL_MAP_ATTRIBUTE = "publish_voxel_map";
    public static final String LETHAL_COST_THRESHOLD_ATTRIBUTE = "lethal_cost_threshold";
    public static final String MAP_TOPIC_ATTRIBUTE = "map_topic";
    public static final String MAP_TYPE_ATTRIBUTE = "map_type";
    public static final String Z_ORIGIN_ATTRIBUTE = "origin_z";
    public static final String Z_RESOLUTION_ATTRIBUTE = "z_resolution";
    public static final String Z_VOXELS_ATTRIBUTE = "z_voxels";
    public static final String UNKNOWN_THRESHOLD_ATTRIBUTE = "unknown_threshold";
    public static final String MARK_THRESHOLD_ATTRIBUTE = "mark_threshold";
    public static final String TRACK_UNKNOWN_SPACE_ATTRIBUTE = "track_unknown_space";
    public static final String RESTORE_DEFAULTS_ATTRIBUTE = "restore_defaults";
    
    public static final String IS_GLOBAL_COSTMAP_ATTRIBUTE = "is_global_costmap";
    
    /**
     * Sophisticated logging mechanism.
     */
    private static Logger logger = Logger.getLogger(ROSCostmapConfiguration.class);
    /**
     * Explanation: Specifies the delay in transform (tf) data that is tolerable
     * in seconds. This parameter serves as a safeguard to losing a link in the
     * tf tree while still allowing an amount of latency the user is comfortable
     * with to exist in the system. For example, a transform being 0.2 seconds
     * out-of-date may be tolerable, but a transform being 8 seconds out of date
     * is not. If the tf transform between the coordinate frames specified by
     * the global_frame and robot_base_frame parameters is transform_tolerance
     * seconds older than ros::Time::now(), then the navigation stack will stop
     * the robot. ROS topic name: '<node_name>/transform_tolerance Type and
     * default: double, 0.3 (0.2 in section 3.2 in ROS wiki) Min, Max value:
     * 0.0, 10.0
     */
    private double transformTolerance;
    /**
     * Explanation: The frequency in Hz for the map to be updated. ROS topic
     * name: '<node_name>/update_frequency' Type and default: double, 5.0 Min,
     * Max value: 0.0, 100.0
     */
    private double updateFrequency;
    /**
     * Explanation: The frequency in Hz for the map to publish display
     * information. ROS topic name: '<node_name>/publish_frequency' Type and
     * default: double, 2.0 (0.0 in section 3.2 in ROS wiki) Min, Max value:
     * 0.0, 100.0
     */
    private double publishFrequency;
    /**
     * Explanation: The maximum height in meters of a sensor reading considered
     * valid. This is usually set to be slightly higher than the height of the
     * robot. ROS topic name: '<node_name>/max_obstacle_height' Type and default:
     * double, 2.0 Min, Max value: 0.0, 50.0
     */
    private double maxObstacleHeight;
    /**
     * Explanation: The maximum range in meters at which to insert obstacles
     * into the costmap using sensor data. ROS topic name:
     * '<node_name>/max_obstacle_range' Type and default: double, 2.5 Min, Max
     * value: 0.0, 50.0
     */
    private double maxObstacleRange;
    /**
     * Explanation: The maximum range in meters at which to raytrace out
     * obstacles from the map using sensor data. This can be overridden on a
     * per-sensor basis. ROS topic name: '<node_name>/raytrace_range' Type and
     * default: double, 3.0 Min, Max value: 0.0, 50.0
     */
    private double raytraceRange;
    /**
     * Explanation: A scaling factor to apply to cost values during inflation.
     * The cost function is computed as follows for all cells in the costmap
     * further than the inscribed radius distance and closer than the inflation
     * radius distance away from an actual obstacle: exp(-1.0 *
     * cost_scaling_factor * (distance_from_obstacle - inscribed_radius)) *
     * (costmap_2d::INSCRIBED_INFLATED_OBSTACLE - 1) where
     * costmap_2d::INSCRIBED_INFLATED_OBSTACLE is currently 254. ROS topic name:
     * '<node_name>/cost_scaling_factor' Type and default: double, 10.0 Min, Max
     * value: 0.0, 100.0
     */
    private double costScalingFactor;
    /**
     * Explanation: The radius in meters to which the map inflates obstacle cost
     * values. ROS topic name: '<node_name>/inflation_radius' Type and default:
     * double, 0.45 (0.55 in section 3.2 in ROS wiki) Min, Max value: 0.0, 50.0
     */
    private double inflationRadius;
    /**
     * Explanation: The footprint of the robot specified in the robot_base_frame
     * coordinate frame as a list in the format: [[x1,y1],[x2,y2],...,[xn,yn]].
     * The footprint specification assumes the center point of the robot is at
     * (0.0, 0.0) in the robot_base_frame and that the points are specified in
     * meters, both clockwise and counter-clockwise orderings of points are
     * supported. ROS topic name: '<node_name>/footprint' Type and default:
     * string, '[]'
     */
    private String footprint;
    /**
     * This parameter is overwritten if the "footprint" parameter is set.
     * Explanation: The radius of the robot in meters , this parameter should
     * only be set for circular robots, all others should use the "footprint"
     * parameter described above. ROS topic name: '<node_name>/robot_radius' Type
     * and default: double, 0.30 (0.46 in section 3.2 in ROS wiki) Min, Max
     * value: 0.0, 10.0
     */
    private double robotRadius;
    /*
     * Explanation: Whether or not to use the static map to initialize the
     * costmap. If the rolling_window parameter is set to true, this parameter
     * must be set to false. ROS topic name: '<node_name>/static_map' Type and
     * default: bool, false (true in section 3.2 in ROS wiki)
     */
    private boolean staticMap;
    /**
     * Explanation: Whether or not to use a rolling window version of the
     * costmap. If the static_map parameter is set to true, this parameter must
     * be set to false. ROS topic name: '<node_name>/rolling_window' Type and
     * default: bool, true (false in section 3.2 in ROS wiki)
     */
    private boolean rollingWindow;
    /**
     * Explanation: The value for which a cost should be considered unknown when
     * reading in a map from the map server. If the costmap is not tracking
     * unknown space, costs of this value will be considered occupied. A value
     * of zero also results in this parameter being unused. ROS topic name:
     * '<node_name>/unknown_cost_value' Type and default: int, 0 Min, Max value:
     * 0, 255
     */
    private int unknownCostValue;
    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The width of the map in meters. ROS topic name:
     * '<node_name>/width' Type and default: int, 6 (10 in section 3.2 in ROS
     * wiki) Min, Max value: 0, 20
     */
    private int width;
    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The height of the map in meters. ROS topic name:
     * '<node_name>/height' Type and default: int, 6 (10 in section 3.2 in ROS
     * wiki) Min, Max value: 0, 20
     */
    private int height;
    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The resolution of the map in meters/cell. ROS topic name:
     * '<node_name>/resolution' Type and default: double, 0.05 Min, Max value:
     * 0.0, 50.0
     */
    private double resolution;
    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The x origin of the map in the global frame in meters. ROS
     * topic name: '<node_name>/origin_x' Type and default: double, 0.0 Min, Max
     * value: 0.0, inf
     */
    private double originX;
    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The y origin of the map in the global frame in meters. ROS
     * topic name: '<node_name>/origin_y' Type and default: double, 0.0 Min, Max
     * value: -inf, inf
     */
    private double originY;
    /**
     * Explanation: Whether or not to publish the underlying voxel grid for
     * visualization purposes. ROS topic name: '<node_name>/publish_voxel_map'
     * Type and default: bool, false
     */
    private boolean publishVoxelMap;
    /**
     * Explanation: The threshold value at which to consider a cost lethal when
     * reading in a map from the map server. ROS topic name:
     * '<node_name>/lethal_cost_threshold' Type and default: int, 100 Min, Max
     * value: 0, 255
     */
    private int lethalCostThreshold;
    /**
     * Explanation: The topic that the costmap subscribes to for the static map.
     * This parameter is useful when you have multiple costmap instances within
     * a single node that you want to use different static maps. ROS topic name:
     * '<node_name>/map_topic'. Type and default: string, 'map'
     */
    private String mapTopic;
    /**
     * Explanation: What map type to use. "voxel" or "costmap" are the supported
     * types, with the difference between them being a 3D-view of the world vs.
     * a 2D-view of the world. ROS topic name: '<node_name>/map_type'. Type and
     * default: string, 'costmap' ('voxel' in section 3.2 in ROS wiki)
     */
    private String mapType;
    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The z origin of the map in meters. ROS topic name:
     * '<node_name>/origin_z'. Type and default: double, 0.0 Min, Max value:
     * 0.0, inf
     */
    private double zOrigin;
    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The z resolution of the map in meters/cell. ROS topic name:
     * '<node_name>/z_resolution'. Type and default: double, 0.2 Min, Max value:
     * 0.0, 50.0
     */
    private double zResolution;
    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The number of voxels to in each vertical column, the height of the grid
     * is z_resolution * z_voxels. ROS topic name: '<node_name>/z_voxels'. Type
     * and default: int, 10 Min, Max value: 0, 16
     */
    private int zVoxels;
    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The number of unknown cells allowed in a column considered to be "known".
     * ROS topic name: '<node_name>/unknown_threshold'. Type and default: int,
     * 15 Min, Max value: 0, 16
     */
    private int unknownThreshold;
    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The maximum number of marked cells allowed in a column considered to be
     * "free". ROS topic name: '<node_name>/mark_threshold'. Type and default:
     * int, 0 Min, Max value: 0, 16
     */
    private int markThreshold;
    /**
     * This parameter is only used if map_type is set to "costmap". Explanation:
     * Specifies whether or not to track what space in the costmap is unknown,
     * meaning that no observation about a cell has been seen from any sensor
     * source. ROS topic name: '<node_name>/track_unknown_space'. Type and
     * default: bool, false
     */
    private boolean trackUnknownSpace;
    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     */
    private boolean restoreDefaults;
    /**
     * Non ROS configuratin parameter! Setting this parameter to true allows for
     * manipulation of global_costmap parameters. Setting this parameter to
     * false allows for manipulation of local_costmap parameters. The default
     * value is 'true'.
     */
    private boolean isGlobalCostmap = true;

    /**
     * ROSCostmapConfiguration constructor. Make sure the member variables are
     * set with the current configuration as is on the corresponding ROS server.
     */
    public ROSCostmapConfiguration() {
        // Fill all the properties with values from the current configuration as
        // is on the ROS server.
        // DO THIS IN BonSAI                
    }

    /**
     * ROSCostmapConfiguration constructor. Make sure the member variables are
     * set with the current configuration as is on the corresponding ROS server.
     *
     * @param isGlobalCostmap Setting this parameter to true allows for
     * manipulation of global_costmap parameters. Setting this parameter to
     * false allows for manipulation of local_costmap parameters. The default
     * value is 'true'.
     */
    public ROSCostmapConfiguration(boolean isGlobalCostmap) {
        this.isGlobalCostmap = isGlobalCostmap;
    }

    /**
     * Explanation: Specifies the delay in transform (tf) data that is tolerable
     * in seconds. This parameter serves as a safeguard to losing a link in the
     * tf tree while still allowing an amount of latency the user is comfortable
     * with to exist in the system. For example, a transform being 0.2 seconds
     * out-of-date may be tolerable, but a transform being 8 seconds out of date
     * is not. If the tf transform between the coordinate frames specified by
     * the global_frame and robot_base_frame parameters is transform_tolerance
     * seconds older than ros::Time::now(), then the navigation stack will stop
     * the robot. ROS topic name: '<node_name>/transform_tolerance Type and
     * default: double, 0.3 (0.2 in section 3.2 in ROS wiki) Min, Max value:
     * 0.0, 10.0
     *
     * @return the transform_tolerance
     */
    public double getTransformTolerance() {
        return transformTolerance;
    }

    /**
     * Explanation: Specifies the delay in transform (tf) data that is tolerable
     * in seconds. This parameter serves as a safeguard to losing a link in the
     * tf tree while still allowing an amount of latency the user is comfortable
     * with to exist in the system. For example, a transform being 0.2 seconds
     * out-of-date may be tolerable, but a transform being 8 seconds out of date
     * is not. If the tf transform between the coordinate frames specified by
     * the global_frame and robot_base_frame parameters is transform_tolerance
     * seconds older than ros::Time::now(), then the navigation stack will stop
     * the robot. ROS topic name: '<node_name>/transform_tolerance Type and
     * default: double, 0.3 (0.2 in section 3.2 in ROS wiki) Min, Max value:
     * 0.0, 10.0
     *
     * @param transform_tolerance the transform_tolerance to set
     */
    public void setTransformTolerance(double transform_tolerance) {
        this.transformTolerance = transform_tolerance;
    }

    /**
     * Explanation: The frequency in Hz for the map to be updated. ROS topic
     * name: '<node_name>/update_frequency Type and default: double, 5.0 Min,
     * Max value: 0.0, 100.0
     *
     * @return the update_frequency
     */
    public double getUpdateFrequency() {
        return updateFrequency;
    }

    /**
     * Explanation: The frequency in Hz for the map to be updated. ROS topic
     * name: '<node_name>/update_frequency Type and default: double, 5.0 Min,
     * Max value: 0.0, 100.0
     *
     * @param update_frequency the update_frequency to set
     */
    public void setUpdateFrequency(double update_frequency) {
        this.updateFrequency = update_frequency;
    }

    /**
     * Explanation: The frequency in Hz for the map to publish display
     * information. ROS topic name: '<node_name>/publish_frequency Type and
     * default: double, 2.0 (0.0 in section 3.2 in ROS wiki) Min, Max value:
     * 0.0, 100.0
     *
     * @return the publish_frequency
     */
    public double getPublishFrequency() {
        return publishFrequency;
    }

    /**
     * Explanation: The frequency in Hz for the map to publish display
     * information. ROS topic name: '<node_name>/publish_frequency Type and
     * default: double, 2.0 (0.0 in section 3.2 in ROS wiki) Min, Max value:
     * 0.0, 100.0
     *
     * @param publish_frequency the publish_frequency to set
     */
    public void setPublishFrequency(double publish_frequency) {
        this.publishFrequency = publish_frequency;
    }

    /**
     * Explanation: The maximum height in meters of a sensor reading considered
     * valid. This is usually set to be slightly higher than the height of the
     * robot. ROS topic name: '<node_name>/max_obstacle_height Type and default:
     * double, 2.0 Min, Max value: 0.0, 50.0
     *
     * @return the max_obstacle_height
     */
    public double getMaxObstacleHeight() {
        return maxObstacleHeight;
    }

    /**
     * Explanation: The maximum height in meters of a sensor reading considered
     * valid. This is usually set to be slightly higher than the height of the
     * robot. ROS topic name: '<node_name>/max_obstacle_height Type and default:
     * double, 2.0 Min, Max value: 0.0, 50.0
     *
     * @param max_obstacle_height the max_obstacle_height to set
     */
    public void setMaxObstacleHeight(double max_obstacle_height) {
        this.maxObstacleHeight = max_obstacle_height;
    }

    /**
     * Explanation: The maximum range in meters at which to insert obstacles
     * into the costmap using sensor data. ROS topic name:
     * '<node_name>/max_obstacle_range Type and default: double, 2.5 Min, Max
     * value: 0.0, 50.0
     *
     * @return the max_obstacle_range
     */
    public double getMaxObstacleRange() {
        return maxObstacleRange;
    }

    /**
     * Explanation: The maximum range in meters at which to insert obstacles
     * into the costmap using sensor data. ROS topic name:
     * '<node_name>/max_obstacle_range Type and default: double, 2.5 Min, Max
     * value: 0.0, 50.0
     *
     * @param max_obstacle_range the max_obstacle_range to set
     */
    public void setMaxObstacleRange(double max_obstacle_range) {
        this.maxObstacleRange = max_obstacle_range;
    }

    /**
     * Explanation: The maximum range in meters at which to raytrace out
     * obstacles from the map using sensor data. This can be overridden on a
     * per-sensor basis. ROS topic name: '<node_name>/raytrace_range Type and
     * default: double, 3.0 Min, Max value: 0.0, 50.0
     *
     * @return the raytrace_range
     */
    public double getRaytraceRange() {
        return raytraceRange;
    }

    /**
     * Explanation: The maximum range in meters at which to raytrace out
     * obstacles from the map using sensor data. This can be overridden on a
     * per-sensor basis. ROS topic name: '<node_name>/raytrace_range Type and
     * default: double, 3.0 Min, Max value: 0.0, 50.0
     *
     * @param raytrace_range the raytrace_range to set
     */
    public void setRaytraceRange(double raytrace_range) {
        this.raytraceRange = raytrace_range;
    }

    /**
     * Explanation: A scaling factor to apply to cost values during inflation.
     * The cost function is computed as follows for all cells in the costmap
     * further than the inscribed radius distance and closer than the inflation
     * radius distance away from an actual obstacle: exp(-1.0 *
     * cost_scaling_factor * (distance_from_obstacle - inscribed_radius)) *
     * (costmap_2d::INSCRIBED_INFLATED_OBSTACLE - 1) where
     * costmap_2d::INSCRIBED_INFLATED_OBSTACLE is currently 254. ROS topic name:
     * '<node_name>/cost_scaling_factor Type and default: double, 10.0 Min, Max
     * value: 0.0, 100.0
     *
     * @return the cost_scaling_factor
     */
    public double getCostScalingFactor() {
        return costScalingFactor;
    }

    /**
     * Explanation: A scaling factor to apply to cost values during inflation.
     * The cost function is computed as follows for all cells in the costmap
     * further than the inscribed radius distance and closer than the inflation
     * radius distance away from an actual obstacle: exp(-1.0 *
     * cost_scaling_factor * (distance_from_obstacle - inscribed_radius)) *
     * (costmap_2d::INSCRIBED_INFLATED_OBSTACLE - 1) where
     * costmap_2d::INSCRIBED_INFLATED_OBSTACLE is currently 254. ROS topic name:
     * '<node_name>/cost_scaling_factor Type and default: double, 10.0 Min, Max
     * value: 0.0, 100.0
     *
     * @param cost_scaling_factor the cost_scaling_factor to set
     */
    public void setCostScalingFactor(double cost_scaling_factor) {
        this.costScalingFactor = cost_scaling_factor;
    }

    /**
     * Explanation: The radius in meters to which the map inflates obstacle cost
     * values. ROS topic name: '<node_name>/inflation_radius Type and default:
     * double, 0.45 (0.55 in section 3.2 in ROS wiki) Min, Max value: 0.0, 50.0
     *
     * @return the inflation_radius
     */
    public double getInflationRadius() {
        return inflationRadius;
    }

    /**
     * Explanation: The radius in meters to which the map inflates obstacle cost
     * values. ROS topic name: '<node_name>/inflation_radius Type and default:
     * double, 0.45 (0.55 in section 3.2 in ROS wiki) Min, Max value: 0.0, 50.0
     *
     * @param inflation_radius the inflation_radius to set
     */
    public void setInflationRadius(double inflation_radius) {
        this.inflationRadius = inflation_radius;
    }

    /**
     * Explanation: The footprint of the robot specified in the robot_base_frame
     * coordinate frame as a list in the format: [[x1,y1],[x2,y2],...,[xn,yn]].
     * The footprint specification assumes the center point of the robot is at
     * (0.0, 0.0) in the robot_base_frame and that the points are specified in
     * meters, both clockwise and counter-clockwise orderings of points are
     * supported. ROS topic name: '<node_name>/footprint Type and default:
     * string, '[]'
     *
     * @return the footprint
     */
    public String getFootprint() {
        return footprint;
    }

    /**
     * Explanation: The footprint of the robot specified in the robot_base_frame
     * coordinate frame as a list in the format: [[x1,y1],[x2,y2],...,[xn,yn]].
     * The footprint specification assumes the center point of the robot is at
     * (0.0, 0.0) in the robot_base_frame and that the points are specified in
     * meters, both clockwise and counter-clockwise orderings of points are
     * supported. ROS topic name: '<node_name>/footprint Type and default:
     * string, '[]'
     *
     * @param footprint the footprint to set
     */
    public void setFootprint(String footprint) {
        this.footprint = footprint;
    }

    /**
     * This parameter is overwritten if the "footprint" parameter is set.
     * Explanation: The radius of the robot in meters , this parameter should
     * only be set for circular robots, all others should use the "footprint"
     * parameter described above. ROS topic name: '<node_name>/robot_radius Type
     * and default: double, 0.30 (0.46 in section 3.2 in ROS wiki) Min, Max
     * value: 0.0, 10.0
     *
     * @return the robot_radius
     */
    public double getRobotRadius() {
        return robotRadius;
    }

    /**
     * This parameter is overwritten if the "footprint" parameter is set.
     * Explanation: The radius of the robot in meters , this parameter should
     * only be set for circular robots, all others should use the "footprint"
     * parameter described above. ROS topic name: '<node_name>/robot_radius Type
     * and default: double, 0.30 (0.46 in section 3.2 in ROS wiki) Min, Max
     * value: 0.0, 10.0
     *
     * @param robot_radius the robot_radius to set
     */
    public void setRobotRadius(double robot_radius) {
        this.robotRadius = robot_radius;
    }

    /**
     * @return the static_map
     */
    public boolean isStaticMap() {
        return staticMap;
    }

    /**
     * @param static_map the static_map to set
     */
    public void setStaticMap(boolean static_map) {
        this.staticMap = static_map;
    }

    /**
     * Explanation: Whether or not to use a rolling window version of the
     * costmap. If the static_map parameter is set to true, this parameter must
     * be set to false. ROS topic name: '<node_name>/rolling_window Type and
     * default: bool, true (false in section 3.2 in ROS wiki)
     *
     * @return the rolling_window
     */
    public boolean isRollingWindow() {
        return rollingWindow;
    }

    /**
     * Explanation: Whether or not to use a rolling window version of the
     * costmap. If the static_map parameter is set to true, this parameter must
     * be set to false. ROS topic name: '<node_name>/rolling_window Type and
     * default: bool, true (false in section 3.2 in ROS wiki)
     *
     * @param rolling_window the rolling_window to set
     */
    public void setRollingWindow(boolean rolling_window) {
        this.rollingWindow = rolling_window;
    }

    /**
     * Explanation: The value for which a cost should be considered unknown when
     * reading in a map from the map server. If the costmap is not tracking
     * unknown space, costs of this value will be considered occupied. A value
     * of zero also results in this parameter being unused. ROS topic name:
     * '<node_name>/unknown_cost_value Type and default: int, 0 Min, Max value:
     * 0, 255
     *
     * @return the unknown_cost_value
     */
    public int getUnknownCostValue() {
        return unknownCostValue;
    }

    /**
     * Explanation: The value for which a cost should be considered unknown when
     * reading in a map from the map server. If the costmap is not tracking
     * unknown space, costs of this value will be considered occupied. A value
     * of zero also results in this parameter being unused. ROS topic name:
     * '<node_name>/unknown_cost_value Type and default: int, 0 Min, Max value:
     * 0, 255
     *
     * @param unknown_cost_value the unknown_cost_value to set
     */
    public void setUnknownCostValue(int unknown_cost_value) {
        this.unknownCostValue = unknown_cost_value;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The width of the map in meters. ROS topic name:
     * '<node_name>/width Type and default: int, 6 (10 in section 3.2 in ROS
     * wiki) Min, Max value: 0, 20
     *
     * @return the width
     */
    public int getWidth() {
        return width;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The width of the map in meters. ROS topic name:
     * '<node_name>/width Type and default: int, 6 (10 in section 3.2 in ROS
     * wiki) Min, Max value: 0, 20
     *
     * @param width the width to set
     */
    public void setWidth(int width) {
        this.width = width;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The height of the map in meters. ROS topic name:
     * '<node_name>/height Type and default: int, 6 (10 in section 3.2 in ROS
     * wiki) Min, Max value: 0, 20
     *
     * @return the height
     */
    public int getHeight() {
        return height;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The height of the map in meters. ROS topic name:
     * '<node_name>/height Type and default: int, 6 (10 in section 3.2 in ROS
     * wiki) Min, Max value: 0, 20
     *
     * @param height the height to set
     */
    public void setHeight(int height) {
        this.height = height;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The resolution of the map in meters/cell. ROS topic name:
     * '<node_name>/resolution Type and default: double, 0.05 Min, Max value:
     * 0.0, 50.0
     *
     * @return the resolution
     */
    public double getResolution() {
        return resolution;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The resolution of the map in meters/cell. ROS topic name:
     * '<node_name>/resolution Type and default: double, 0.05 Min, Max value:
     * 0.0, 50.0
     *
     * @param resolution the resolution to set
     */
    public void setResolution(double resolution) {
        this.resolution = resolution;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The x origin of the map in the global frame in meters. ROS
     * topic name: '<node_name>/origin_x Type and default: double, 0.0 Min, Max
     * value: 0.0, inf
     *
     * @return the origin_x
     */
    public double getOriginX() {
        return originX;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The x origin of the map in the global frame in meters. ROS
     * topic name: '<node_name>/origin_x Type and default: double, 0.0 Min, Max
     * value: 0.0, inf
     *
     * @param origin_x the origin_x to set
     */
    public void setOriginX(double origin_x) {
        this.originX = origin_x;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The y origin of the map in the global frame in meters. ROS
     * topic name: '<node_name>/origin_y Type and default: double, 0.0 Min, Max
     * value: -inf, inf
     *
     * @return the origin_y
     */
    public double getOriginY() {
        return originY;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The y origin of the map in the global frame in meters. ROS
     * topic name: '<node_name>/origin_y Type and default: double, 0.0 Min, Max
     * value: -inf, inf
     *
     * @param origin_y the origin_y to set
     */
    public void setOriginY(double origin_y) {
        this.originY = origin_y;
    }

    /**
     * Explanation: Whether or not to publish the underlying voxel grid for
     * visualization purposes. ROS topic name: '<node_name>/publish_voxel_map
     * Type and default: bool, false
     *
     * @return the publishVoxelMap
     */
    public boolean isPublishVoxelMap() {
        return publishVoxelMap;
    }

    /**
     * Explanation: Whether or not to publish the underlying voxel grid for
     * visualization purposes. ROS topic name: '<node_name>/publish_voxel_map
     * Type and default: bool, false
     *
     * @param publishVoxelMap the publishVoxelMap to set
     */
    public void setPublishVoxelMap(boolean publishVoxelMap) {
        this.publishVoxelMap = publishVoxelMap;
    }

    /**
     * Explanation: The threshold value at which to consider a cost lethal when
     * reading in a map from the map server. ROS topic name:
     * '<node_name>/lethal_cost_threshold Type and default: int, 100 Min, Max
     * value: 0, 255
     *
     * @return the lethalCostThreshold
     */
    public int getLethalCostThreshold() {
        return lethalCostThreshold;
    }

    /**
     * Explanation: The threshold value at which to consider a cost lethal when
     * reading in a map from the map server. ROS topic name:
     * '<node_name>/lethal_cost_threshold Type and default: int, 100 Min, Max
     * value: 0, 255
     *
     * @param lethalCostThreshold the lethalCostThreshold to set
     */
    public void setLethalCostThreshold(int lethalCostThreshold) {
        this.lethalCostThreshold = lethalCostThreshold;
    }

    /**
     * Explanation: The topic that the costmap subscribes to for the static map.
     * This parameter is useful when you have multiple costmap instances within
     * a single node that you want to use different static maps. ROS topic name:
     * '<node_name>/map_topic'. Type and default: string, 'map'
     *
     * @return the mapTopic
     */
    public String getMapTopic() {
        return mapTopic;
    }

    /**
     * Explanation: The topic that the costmap subscribes to for the static map.
     * This parameter is useful when you have multiple costmap instances within
     * a single node that you want to use different static maps. ROS topic name:
     * '<node_name>/map_topic'. Type and default: string, 'map'
     *
     * @param mapTopic the mapTopic to set
     */
    public void setMapTopic(String mapTopic) {
        this.mapTopic = mapTopic;
    }

    /**
     * Explanation: What map type to use. "voxel" or "costmap" are the supported
     * types, with the difference between them being a 3D-view of the world vs.
     * a 2D-view of the world. ROS topic name: '<node_name>/map_type'. Type and
     * default: string, 'costmap' ('voxel' in section 3.2 in ROS wiki)
     *
     * @return the mapType
     */
    public String getMapType() {
        return mapType;
    }

    /**
     * Explanation: What map type to use. "voxel" or "costmap" are the supported
     * types, with the difference between them being a 3D-view of the world vs.
     * a 2D-view of the world. ROS topic name: '<node_name>/map_type'. Type and
     * default: string, 'costmap' ('voxel' in section 3.2 in ROS wiki)
     *
     * @param mapType the mapType to set
     */
    public void setMapType(String mapType) {
        this.mapType = mapType;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The z origin of the map in meters. ROS topic name:
     * '<node_name>/origin_z'. Type and default: double, 0.0 Min, Max value:
     * 0.0, inf
     *
     * @return the zOrigin
     */
    public double getzOrigin() {
        return zOrigin;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The z origin of the map in meters. ROS topic name:
     * '<node_name>/origin_z'. Type and default: double, 0.0 Min, Max value:
     * 0.0, inf
     *
     * @param zOrigin the zOrigin to set
     */
    public void setzOrigin(double zOrigin) {
        this.zOrigin = zOrigin;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The z resolution of the map in meters/cell. ROS topic name:
     * '<node_name>/z_resolution'. Type and default: double, 0.2 Min, Max value:
     * 0.0, 50.0
     *
     * @return the zResolution
     */
    public double getzResolution() {
        return zResolution;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The z resolution of the map in meters/cell. ROS topic name:
     * '<node_name>/z_resolution'. Type and default: double, 0.2 Min, Max value:
     * 0.0, 50.0
     *
     * @param zResolution the zResolution to set
     */
    public void setzResolution(double zResolution) {
        this.zResolution = zResolution;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The number of voxels to in each vertical column, the height of the grid
     * is z_resolution * z_voxels. ROS topic name: '<node_name>/z_voxels'. Type
     * and default: int, 10 Min, Max value: 0, 16
     *
     * @return the zVoxels
     */
    public int getzVoxels() {
        return zVoxels;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The number of voxels to in each vertical column, the height of the grid
     * is z_resolution * z_voxels. ROS topic name: '<node_name>/z_voxels'. Type
     * and default: int, 10 Min, Max value: 0, 16
     *
     * @param zVoxels the zVoxels to set
     */
    public void setzVoxels(int zVoxels) {
        this.zVoxels = zVoxels;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The number of unknown cells allowed in a column considered to be "known".
     * ROS topic name: '<node_name>/unknown_threshold'. Type and default: int,
     * 15 Min, Max value: 0, 16
     *
     * @return the unknownThreshold
     */
    public int getUnknownThreshold() {
        return unknownThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The number of unknown cells allowed in a column considered to be "known".
     * ROS topic name: '<node_name>/unknown_threshold'. Type and default: int,
     * 15 Min, Max value: 0, 16
     *
     * @param unknownThreshold the unknownThreshold to set
     */
    public void setUnknownThreshold(int unknownThreshold) {
        this.unknownThreshold = unknownThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The maximum number of marked cells allowed in a column considered to be
     * "free". ROS topic name: '<node_name>/mark_threshold'. Type and default:
     * int, 0 Min, Max value: 0, 16
     *
     * @return the markThreshold
     */
    public int getMarkThreshold() {
        return markThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "voxel". Explanation:
     * The maximum number of marked cells allowed in a column considered to be
     * "free". ROS topic name: '<node_name>/mark_threshold'. Type and default:
     * int, 0 Min, Max value: 0, 16
     *
     * @param markThreshold the markThreshold to set
     */
    public void setMarkThreshold(int markThreshold) {
        this.markThreshold = markThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "costmap". Explanation:
     * Specifies whether or not to track what space in the costmap is unknown,
     * meaning that no observation about a cell has been seen from any sensor
     * source. ROS topic name: '<node_name>/track_unknown_space'. Type and
     * default: bool, false
     *
     * @return the trackUnknownSpace
     */
    public boolean isTrackUnknownSpace() {
        return trackUnknownSpace;
    }

    /**
     * This parameter is only used if map_type is set to "costmap". Explanation:
     * Specifies whether or not to track what space in the costmap is unknown,
     * meaning that no observation about a cell has been seen from any sensor
     * source. ROS topic name: '<node_name>/track_unknown_space'. Type and
     * default: bool, false
     *
     * @param trackUnknownSpace the trackUnknownSpace to set
     */
    public void setTrackUnknownSpace(boolean trackUnknownSpace) {
        this.trackUnknownSpace = trackUnknownSpace;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     *
     * @return the restoreDefaults
     */
    public boolean isRestoreDefaults() {
        return restoreDefaults;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults. ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     *
     * @param restoreDefaults the restoreDefaults to set
     */
    public void setRestoreDefaults(boolean restoreDefaults) {
        this.restoreDefaults = restoreDefaults;
    }

    /**
     * Non ROS configuratin parameter! Setting this parameter to true allows for
     * manipulation of global_costmap parameters. Setting this parameter to
     * false allows for manipulation of local_costmap parameters. The default
     * value is 'true'.
     *
     * @return the isGlobalCostmap
     */
    public boolean isIsGlobalCostmap() {
        return isGlobalCostmap;
    }

    /**
     * Non ROS configuratin parameter! Setting this parameter to true allows for
     * manipulation of global_costmap parameters. Setting this parameter to
     * false allows for manipulation of local_costmap parameters. The default
     * value is 'true'.
     *
     * @param isGlobalCostmap the isGlobalCostmap to set
     */
    public void setIsGlobalCostmap(boolean isGlobalCostmap) {
        this.isGlobalCostmap = isGlobalCostmap;
    }

    /**
     * Fills an {@link ROSCostmapConfiguration} object from a given XOM {@link Element}
     *
     * @param objectElement The XOM {@link Element} to fill an object from.
     * @param type The {@link ROSCostmapConfiguration} object to fill with all
     * the information given by the {@link Element} object.
     */
    public static void fromElement(Element objectElement, ROSCostmapConfiguration type)
            throws ParsingException {

        Type.fromElement(objectElement, type);

        try {

            Nodes positionNodes = objectElement.query(COSTMAP_CONFIGURATION_TAG);
            if (positionNodes.size() < 1
                    && !(positionNodes.get(0) instanceof Element)) {
                throw new ParsingException(
                        "Unable to find element with name " + COSTMAP_CONFIGURATION_TAG + ".");
            }

            Element temp = (Element) positionNodes.get(0);

            type.transformTolerance = Double.parseDouble(temp.getAttribute(TRANSFORM_TOLERANCE_ATTRIBUTE).getValue());
            type.updateFrequency = Double.parseDouble(temp.getAttribute(UPDATE_FREQUENCY_ATTRIBUTE).getValue());
            type.publishFrequency = Double.parseDouble(temp.getAttribute(PUBLISH_FREQUENCY_ATTRIBUTE).getValue());
            type.maxObstacleHeight = Double.parseDouble(temp.getAttribute(MAX_OBSTACLE_HEIGHT_ATTRIBUTE).getValue());
            type.maxObstacleRange = Double.parseDouble(temp.getAttribute(MAX_OBSTACLE_RANGE_ATTRIBUTE).getValue());
            type.raytraceRange = Double.parseDouble(temp.getAttribute(RAYTRACE_RANGE_ATTRIBUTE).getValue());
            type.costScalingFactor = Double.parseDouble(temp.getAttribute(COST_SCALING_FACTOR_ATTRIBUTE).getValue());
            type.inflationRadius = Double.parseDouble(temp.getAttribute(INFLATION_RADIUS_ATTRIBUTE).getValue());
            type.footprint = temp.getAttribute(FOOTPRINT_ATTRIBUTE).getValue();
            type.robotRadius = Double.parseDouble(temp.getAttribute(ROBOT_RADIUS_ATTRIBUTE).getValue());            
            type.staticMap = Boolean.parseBoolean(temp.getAttribute(STATIC_MAP_ATTRIBUTE).getValue());
            type.rollingWindow = Boolean.parseBoolean(temp.getAttribute(ROLLING_WINDOW_ATTRIBUTE).getValue());
            type.unknownCostValue = Integer.parseInt(temp.getAttribute(UNKNOWN_COST_VALUE_ATTRIBUTE).getValue());
            type.width = Integer.parseInt(temp.getAttribute(WIDTH_ATTRIBUTE).getValue());
            type.height = Integer.parseInt(temp.getAttribute(HEIGHT_ATTRIBUTE).getValue());
            type.resolution = Double.parseDouble(temp.getAttribute(RESOLUTION_ATTRIBUTE).getValue());
            type.originX = Double.parseDouble(temp.getAttribute(ORIGIN_X_ATTRIBUTE).getValue());
            type.originY = Double.parseDouble(temp.getAttribute(ORIGIN_Y_ATTRIBUTE).getValue());
            type.publishVoxelMap = Boolean.parseBoolean(temp.getAttribute(PUBLISH_VOXEL_MAP_ATTRIBUTE).getValue());
            type.lethalCostThreshold = Integer.parseInt(temp.getAttribute(LETHAL_COST_THRESHOLD_ATTRIBUTE).getValue());
            type.mapTopic = temp.getAttribute(MAP_TOPIC_ATTRIBUTE).getValue();
            type.mapType = temp.getAttribute(MAP_TYPE_ATTRIBUTE).getValue();
            type.zOrigin = Double.parseDouble(temp.getAttribute(Z_ORIGIN_ATTRIBUTE).getValue());
            type.zResolution = Double.parseDouble(temp.getAttribute(Z_RESOLUTION_ATTRIBUTE).getValue());
            type.zVoxels = Integer.parseInt(temp.getAttribute(Z_VOXELS_ATTRIBUTE).getValue());
            type.unknownThreshold = Integer.parseInt(temp.getAttribute(UNKNOWN_THRESHOLD_ATTRIBUTE).getValue());
            type.markThreshold = Integer.parseInt(temp.getAttribute(MARK_THRESHOLD_ATTRIBUTE).getValue());
            type.trackUnknownSpace = Boolean.parseBoolean(temp.getAttribute(TRACK_UNKNOWN_SPACE_ATTRIBUTE).getValue());
            type.restoreDefaults = Boolean.parseBoolean(temp.getAttribute(RESTORE_DEFAULTS_ATTRIBUTE).getValue());
            type.isGlobalCostmap = Boolean.parseBoolean(temp.getAttribute(IS_GLOBAL_COSTMAP_ATTRIBUTE).getValue());

        } catch (NullPointerException ex) {
            // this happens when an element or attribute that is required is
            // not present
            throw new IllegalArgumentException("Missing element or attribute "
                    + "in document.", ex);
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("Can not parse number", e);
        }
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void fillInto(Element parent) {

        sanitizeElement(parent);
        super.fillInto(parent);

        Element configuration = new Element(COSTMAP_CONFIGURATION_TAG);

        configuration.addAttribute(new Attribute(COST_SCALING_FACTOR_ATTRIBUTE, String.valueOf(this.costScalingFactor)));        
        configuration.addAttribute(new Attribute(FOOTPRINT_ATTRIBUTE, String.valueOf(this.footprint)));
        configuration.addAttribute(new Attribute(HEIGHT_ATTRIBUTE, String.valueOf(this.height)));
        configuration.addAttribute(new Attribute(INFLATION_RADIUS_ATTRIBUTE, String.valueOf(this.inflationRadius)));
        configuration.addAttribute(new Attribute(IS_GLOBAL_COSTMAP_ATTRIBUTE, String.valueOf(this.isGlobalCostmap)));
        configuration.addAttribute(new Attribute(LETHAL_COST_THRESHOLD_ATTRIBUTE, String.valueOf(this.lethalCostThreshold)));
        configuration.addAttribute(new Attribute(MAP_TOPIC_ATTRIBUTE, String.valueOf(this.mapTopic)));
        configuration.addAttribute(new Attribute(MAP_TYPE_ATTRIBUTE, String.valueOf(this.mapType)));
        configuration.addAttribute(new Attribute(MARK_THRESHOLD_ATTRIBUTE, String.valueOf(this.markThreshold)));
        configuration.addAttribute(new Attribute(MAX_OBSTACLE_HEIGHT_ATTRIBUTE, String.valueOf(this.maxObstacleHeight)));
        configuration.addAttribute(new Attribute(MAX_OBSTACLE_RANGE_ATTRIBUTE, String.valueOf(this.maxObstacleRange)));
        configuration.addAttribute(new Attribute(RESOLUTION_ATTRIBUTE, String.valueOf(this.resolution)));
        configuration.addAttribute(new Attribute(ORIGIN_X_ATTRIBUTE, String.valueOf(this.originX)));
        configuration.addAttribute(new Attribute(ORIGIN_Y_ATTRIBUTE, String.valueOf(this.originY)));
        configuration.addAttribute(new Attribute(PUBLISH_FREQUENCY_ATTRIBUTE, String.valueOf(this.publishFrequency)));
        configuration.addAttribute(new Attribute(PUBLISH_VOXEL_MAP_ATTRIBUTE, String.valueOf(this.publishVoxelMap)));
        configuration.addAttribute(new Attribute(RAYTRACE_RANGE_ATTRIBUTE, String.valueOf(this.raytraceRange)));
        configuration.addAttribute(new Attribute(RESTORE_DEFAULTS_ATTRIBUTE, String.valueOf(this.restoreDefaults)));
        configuration.addAttribute(new Attribute(ROBOT_RADIUS_ATTRIBUTE, String.valueOf(this.robotRadius)));
        configuration.addAttribute(new Attribute(ROLLING_WINDOW_ATTRIBUTE, String.valueOf(this.rollingWindow)));
        configuration.addAttribute(new Attribute(STATIC_MAP_ATTRIBUTE, String.valueOf(this.staticMap)));
        configuration.addAttribute(new Attribute(TRACK_UNKNOWN_SPACE_ATTRIBUTE, String.valueOf(this.trackUnknownSpace)));
        configuration.addAttribute(new Attribute(TRANSFORM_TOLERANCE_ATTRIBUTE, String.valueOf(this.transformTolerance)));
        configuration.addAttribute(new Attribute(UNKNOWN_COST_VALUE_ATTRIBUTE, String.valueOf(this.unknownCostValue)));
        configuration.addAttribute(new Attribute(UNKNOWN_THRESHOLD_ATTRIBUTE, String.valueOf(this.unknownThreshold)));
        configuration.addAttribute(new Attribute(UPDATE_FREQUENCY_ATTRIBUTE, String.valueOf(this.updateFrequency)));
        configuration.addAttribute(new Attribute(WIDTH_ATTRIBUTE, String.valueOf(this.width)));
        configuration.addAttribute(new Attribute(Z_ORIGIN_ATTRIBUTE, String.valueOf(this.zOrigin)));
        configuration.addAttribute(new Attribute(Z_RESOLUTION_ATTRIBUTE, String.valueOf(this.zResolution)));
        configuration.addAttribute(new Attribute(Z_VOXELS_ATTRIBUTE, String.valueOf(this.zVoxels)));
        
        parent.appendChild(configuration);
    }
}
