package de.unibi.airobots.btl.data.navigation;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.MicroTimestamp;
import de.unibi.airobots.btl.data.common.Ticked;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.SpeedUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This class is used to store data from the speed sensors. It holds the
 * translational Speed[m/s], rotational Speed[rad/s] and the time of these
 * sensor values as a timestamp.
 * 
 * @author jwienke
 * @author unknown
 */
public class SpeedData extends Type implements Ticked {

	private static Logger logger = Logger.getLogger(SpeedData.class);

	private double speedTranslation;
	private double speedRotation;
	private MicroTimestamp readTime = new MicroTimestamp();

	private static SpeedUnit iSU = SpeedUnit.METER_PER_SEC;
	private static AngleUnit iAU = AngleUnit.RADIAN;

	/**
	 * @return the translational Speed[SpeedUnit]
	 */
	public double getSpeedTranslation(SpeedUnit u) {
		return UnitConverter.convert(speedTranslation, iSU, u);
	}

	/**
	 * @return the rotational Speed[AngleUnit/s]
	 */
	public double getSpeedRotation(AngleUnit u) {
		return UnitConverter.convert(speedRotation, iAU, u);
	}

	/**
	 * Sets the translational speed.
	 * 
	 * @param speedTranslation
	 *            translational speed in [SpeedUnit]
	 */
	public void setSpeedTranslation(double speedTranslation, SpeedUnit u) {
		this.speedTranslation = UnitConverter.convert(speedTranslation, u, iSU);
	}

	/**
	 * Sets the rotational speed.
	 * 
	 * @param speedRotation
	 *            rotational speed [AngleUnit/s]
	 */
	public void setSpeedRotation(double speedRotation, AngleUnit u) {
		this.speedRotation = UnitConverter.convert(speedRotation, u, iAU);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#SPEED# ");
		strb.append("timestamp: " + getTimestamp() + "; ");
		strb.append("translation: " + speedTranslation + "; ");
		strb.append("rotation: " + speedRotation + ";");
		return strb.toString();
	}

	public static String getBaseTag() {
		return "SPEEDS";
	}

	public static void fromElement(Element element, SpeedData type)
			throws ParsingException {


		Type.fromElement(element, type);

		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("ROTATIONSPEED")) {
					type.setSpeedRotation(ElementParser
							.getDoubleAttributeValue(elem, "quant"), iAU);
				} else if (elem.getLocalName().equals("TRANSLATIONSPEED")) {
					type.setSpeedTranslation(ElementParser
							.getDoubleAttributeValue(elem, "quant"), iSU);
				} else if (elem.getLocalName().equals(
						MicroTimestamp.getBaseTag())) {
					MicroTimestamp readTime = new MicroTimestamp();
					MicroTimestamp.fromElement(elem, readTime);
					type.setReadTime(readTime);
				}
			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		readTime.fillInto(readTimeElement);

		Element rotElement = new Element("ROTATIONSPEED");
		parent.appendChild(rotElement);
		rotElement.addAttribute(new Attribute("quant", String
				.valueOf(getSpeedRotation(iAU))));

		Element transElement = new Element("TRANSLATIONSPEED");
		parent.appendChild(transElement);
		transElement.addAttribute(new Attribute("quant", String
				.valueOf(getSpeedTranslation(iSU))));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;
	}

}
