package de.unibi.airobots.btl.data.object;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.awt.Point;
import java.util.HashSet;
import java.util.Set;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.data.geometry.Polygon;
import de.unibi.airobots.btl.data.object.ObjectData.Hypothesis;

/**
 * Results of the object recognition. This class is meat so define the location
 * of the object in the detector's camera image! The given polygon describes the
 * objects's location in pixel coordinates! If you want to define an object in
 * world coordinates use {@link ObjectPositionData}.
 * 
 * @author lziegler
 */
public class ObjectLocationData extends ObjectData {

	protected final static String SENSORTYPE_ATTRIBUTE = "SENSORTYPE";

	public static class Hypothesis extends ObjectData.Hypothesis {

		public Hypothesis(Hypothesis hyp) {
			super(hyp);
		}

		public Hypothesis() {
			super();
		}

		/**
		 * Getter for the xml base tag used for this (de-)serialization.
		 * 
		 * @return xml base tag
		 */
		public static String getBaseTag() {
			return ObjectData.Hypothesis.getBaseTag();
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void fillInto(Element parent) {
			super.fillInto(parent);
		}

		/**
		 * Fills an {@link ObjectLocationData.Hypothesis} object from a given
		 * XOM {@link Element}.
		 * 
		 * @param hypothesisElement
		 *            The XOM {@link Element} to fill an object from.
		 * @param type
		 *            The {@link ObjectLocationData.Hypothesis} object to fill
		 *            with all the information given by the {@link Element}
		 *            object.
		 */
		public static void fromElement(Element hypothesisElement,
				Hypothesis type) {

			ObjectData.Hypothesis.fromElement(hypothesisElement, type);

		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public String toString() {
			return super.toString();

		}
	}

	/**
	 * Position of the object in the sensor's image.
	 */
	private Polygon polygon;

	public Polygon getPolygon() {
		return polygon;
	}

	public void setPolygon(Polygon polygon) {
		this.polygon = polygon;
	}

	/**
	 * Description of the hardware, that recorded the location.
	 */
	private String sensorType;

	public String getSensorType() {
		return sensorType;
	}

	public void setSensorType(String sensorType) {
		this.sensorType = sensorType;
	}

	/**
	 * Default constructor.
	 */
	public ObjectLocationData() {
		setLegacyParsing(true);
	}

	/**
	 * Copy constructor.
	 */
	public ObjectLocationData(ObjectLocationData data) {
		super(data);
		
		polygon = new Polygon();
		for (Point p : data.polygon) {
			polygon.addPoint(p);
		}
		sensorType = new String(data.sensorType);
		hypotheses.clear();
		for (Hypothesis h : data.getLocationHypotheses()) {
			addHypothesis(new Hypothesis(h));
		}
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "OBJECTLOCATION";
	}

	public Set<Hypothesis> getLocationHypotheses() {

		HashSet<Hypothesis> set = new HashSet<Hypothesis>();
		for (ObjectData.Hypothesis h : hypotheses) {
			if (h instanceof Hypothesis) {
				set.add((Hypothesis) h);
			}
		}
		return set;
	}

	/**
	 * @deprecated This method is not valid for subclasses of {@link ObjectData}
	 *             . Use {@link #getLocationHypotheses()} instead.
	 */
	@Deprecated
	public Set<ObjectData.Hypothesis> getHypotheses() {
		return super.getHypotheses();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// polygon
		Element polygonElem = new Element(Polygon.getBaseTag());
		polygon.fillInto(polygonElem);

		// sensor type
		Element sensorElem = new Element(SENSORTYPE_ATTRIBUTE);
		sensorElem.appendChild(sensorType);

		parent.appendChild(polygonElem);
		parent.appendChild(sensorElem);
	}

	/**
	 * Fills an {@link ObjectLocationData} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData} object to fill with all the
	 *            information given by the {@link Element} object.
	 */
	public static void fromElement(Element objectElement,
			ObjectLocationData type) {

		try {

			ObjectData.fromElement(objectElement, type);

			type.setPolygon(Polygon.fromElement(objectElement
					.getFirstChildElement(Polygon.getBaseTag())));
			type.setSensorType(objectElement.getFirstChildElement(
					SENSORTYPE_ATTRIBUTE).getValue());

			// clean up
			type.clearHypotheses();

			// check hypotheses
			Nodes nodes = objectElement.query(Hypothesis.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {

					Hypothesis hyp = new Hypothesis();
					Hypothesis.fromElement((Element) node, hyp);
					type.addHypothesis(hyp);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String out = super.toString().replace("[OBJECT", "[OBJECTLOCATION");
		out = out.replace("]", "");
		out += " sensor type: " + sensorType + " Polygon: " + polygon + "]";
		return out;
	}
}
