package de.unibi.airobots.btl.data.object;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.HashSet;
import java.util.Set;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * Results of the object recognition. This class is meat so define the location
 * of the object in the detector's camera image and contain shape information in
 * 3D! The given polygon describes the objects's location in pixel coordinates!
 * 
 * @author lziegler
 */
public class ObjectShapeData extends ObjectLocationData {

	private static final String ATTR_WIDTH = "width";
	private static final String ATTR_HEIGHT = "height";
	private static final String ATTR_DEPTH = "depth";
	private static final String TAG_SHAPE = "SHAPE";

	public static class Hypothesis extends ObjectLocationData.Hypothesis {

		public Hypothesis(Hypothesis hyp) {
			super(hyp);
		}

		public Hypothesis() {
			super();
		}

		/**
		 * Getter for the xml base tag used for this (de-)serialization.
		 * 
		 * @return xml base tag
		 */
		public static String getBaseTag() {
			return ObjectLocationData.Hypothesis.getBaseTag();
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void fillInto(Element parent) {
			super.fillInto(parent);
		}

		/**
		 * Fills an {@link ObjectShapeData.Hypothesis} object from a given XOM
		 * {@link Element}.
		 * 
		 * @param hypothesisElement
		 *            The XOM {@link Element} to fill an object from.
		 * @param type
		 *            The {@link ObjectShapeData.Hypothesis} object to fill with
		 *            all the information given by the {@link Element} object.
		 */
		public static void fromElement(Element hypothesisElement,
				Hypothesis type) {

			ObjectLocationData.Hypothesis.fromElement(hypothesisElement, type);
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public String toString() {
			return super.toString();

		}
	}

	public double getEpsilonToOtherObject(ObjectShapeData other) {
		double eps = 0;

		eps += (other.getCenter().distance(this.center));
		eps += Math.abs(this.height - other.height);
		eps += Math.abs(this.width - other.width);
		eps += Math.abs(this.depth - other.depth);

		return eps;
	}

	public ObjectShapeData(ObjectShapeData osd) {
		super(osd);
		this.center = new Point3D(osd.center);
		this.height = osd.height;
		this.width = osd.width;
		this.depth = osd.depth;

		hypotheses.clear();
		for (Hypothesis h : osd.getShapeHypotheses()) {
			addHypothesis(new Hypothesis(h));
		}
	}

	private Point3D center;
	private double height, width, depth;
	private LengthUnit internalUnit = LengthUnit.MILLIMETER;

	public Point3D getCenter() {
		return center;
	}

	public void setCenter(Point3D center) {
		this.center = center;
	}

	public double getHeight(LengthUnit unit) {
		return UnitConverter.convert(height, this.internalUnit, unit);
	}

	public void setHeight(double height, LengthUnit unit) {
		this.height = UnitConverter.convert(height, unit, this.internalUnit);
	}

	public double getWidth(LengthUnit unit) {
		return UnitConverter.convert(width, this.internalUnit, unit);
	}

	public void setWidth(double width, LengthUnit unit) {
		this.width = UnitConverter.convert(width, unit, this.internalUnit);
	}

	public double getDepth(LengthUnit unit) {
		return UnitConverter.convert(depth, this.internalUnit, unit);
	}

	public void setDepth(double depth, LengthUnit unit) {
		this.depth = UnitConverter.convert(depth, unit, this.internalUnit);
	}

	/**
	 * Default constructor.
	 */
	public ObjectShapeData() {
		setLegacyParsing(true);
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "OBJECTSHAPE";
	}

	public Set<Hypothesis> getShapeHypotheses() {

		HashSet<Hypothesis> set = new HashSet<Hypothesis>();
		for (ObjectData.Hypothesis h : hypotheses) {
			if (h instanceof Hypothesis) {
				set.add((Hypothesis) h);
			}
		}
		return set;
	}

	/**
	 * @deprecated This method is not valid for subclasses of
	 *             {@link ObjectLocationData} . Use
	 *             {@link #getShapeHypotheses()} instead.
	 */
	@Override
	@Deprecated
	public Set<ObjectLocationData.Hypothesis> getLocationHypotheses() {
		return super.getLocationHypotheses();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element shapeElem = new Element(TAG_SHAPE);
		shapeElem
				.addAttribute(new Attribute(ATTR_DEPTH, String.valueOf(depth)));
		shapeElem.addAttribute(new Attribute(ATTR_HEIGHT, String
				.valueOf(height)));
		shapeElem
				.addAttribute(new Attribute(ATTR_WIDTH, String.valueOf(width)));
		parent.appendChild(shapeElem);

		Element centerElem = new Element(Point3D.getBaseTag());
		center.fillInto(centerElem);
		parent.appendChild(centerElem);
	}

	/**
	 * Fills an {@link ObjectShapeData} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectShapeData} object to fill with all the
	 *            information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element objectElement, ObjectShapeData type)
			throws ParsingException {

		try {

			ObjectLocationData.fromElement(objectElement, type);

			Element shapeElem = objectElement.getFirstChildElement(TAG_SHAPE);
			type.height = Double.parseDouble(shapeElem
					.getAttribute(ATTR_HEIGHT).getValue());
			type.width = Double.parseDouble(shapeElem.getAttribute(ATTR_WIDTH)
					.getValue());
			type.depth = Double.parseDouble(shapeElem.getAttribute(ATTR_DEPTH)
					.getValue());

			type.center = new Point3D();
			Point3D.fromElement(
					objectElement.getFirstChildElement(Point3D.getBaseTag()),
					type.center);

			// clean up
			type.clearHypotheses();

			// check hypotheses
			Nodes nodes = objectElement.query(Hypothesis.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {

					Hypothesis hyp = new Hypothesis();
					Hypothesis.fromElement((Element) node, hyp);
					type.addHypothesis(hyp);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String out = super.toString().replace("[OBJECT", "[OBJECTSHAPE");
		out = out.replace("]", "");
		out += " center: " + center;
		out += " height: " + height;
		out += " depth: " + depth;
		out += " width: " + width;
		out += "]";
		return out;
	}
}
