package de.unibi.airobots.btl.data.speechrec;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.ArrayList;
import java.util.List;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.MicroTimestamp;
import de.unibi.airobots.btl.data.common.Ticked;

/**
 * BTL type for work with the output from the sploc ES speaker localization
 * application.
 * 
 * @author dschulze
 * @author dklotz
 */
public class SplocAngleData extends Type implements Ticked {

	private static final String ANGLE_TAG = "ANGLE";

	/** A list of all angles computed from the SplocES. */
	private List<Double> angles = new ArrayList<Double>();
	/** The exact timestamp the data was read / computed. */
	private MicroTimestamp readTime = new MicroTimestamp();

	// Constants for all the XML field names
	private static final String BASE_TAG = "SPLOCANGLES";
	private static final String ANGLES_TAG = "ANGLES";
	private static final String VALUE_ATTRIBUTE = "value";

	public static String getBaseTag() {
		return BASE_TAG;
	}

	public void setAngles(List<Double> angles) {
		this.angles = angles;
	}

	public List<Double> getAngles() {
		return angles;
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// Append the exact timestamp
		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		readTime.fillInto(readTimeElement);

		// Append angle information.
		Element anglesElement = new Element(ANGLES_TAG);

		for (Double angleValue : angles) {
			Element angleElement = new Element(ANGLE_TAG);
			angleElement.addAttribute(new Attribute(VALUE_ATTRIBUTE, String
					.valueOf(angleValue)));
			anglesElement.appendChild(angleElement);
		}

		parent.appendChild(anglesElement);

	}

	/** @{inheritDoc */
	public static void fromElement(Element element, SplocAngleData data)
			throws ParsingException {

		Type.fromElement(element, data);

		List<Double> angles = new ArrayList<Double>();

		Element anglesElement = element.getFirstChildElement(ANGLES_TAG);
		Elements angleElements = anglesElement.getChildElements(ANGLE_TAG);

		for (int i = 0; i < angleElements.size(); i++) {
			Element angleElement = angleElements.get(i);
			angles.add(ElementParser.getDoubleAttributeValue(angleElement,
					VALUE_ATTRIBUTE));
		}

		data.setAngles(angles);

		Element readTimeElement = element.getFirstChildElement(MicroTimestamp
				.getBaseTag());
		MicroTimestamp readTime = new MicroTimestamp();
		MicroTimestamp.fromElement(readTimeElement, readTime);
		data.setReadTime(readTime);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;
	}

}
