/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * ObjectPositionDataTest.cpp
 *
 *  Created on: Apr 28, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/object/ObjectPositionData.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ObjectPositionDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		vector<pair<double,double> > polygon;
		polygon.push_back(pair<double,double>(0,0));
		polygon.push_back(pair<double,double>(1,1));
		polygon.push_back(pair<double,double>(2,2));

		ObjectPositionData original;
		original.setPolygon(polygon, MILLIMETER);
		original.setCoordinateKind("absolute");
		original.setReference("world");

		ObjectPositionData::Hypothesis originalHyp;
		originalHyp.setGenerator("test");
		originalHyp.setTimestamp(Timestamp(253543797,253543712));
		originalHyp.setReliability(0.1);
		originalHyp.setClassLabel(string("smacks"));
		original.addHypothesis(originalHyp);

		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, original.getPolygon(CENTIMETER)[2].first * 10, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, original.getPolygon(METER)[2].first * 1000, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, original.getPolygon(MILLIMETER)[2].first, TOLERANCE);
		original.setPolygon(polygon, MILLIMETER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 2.0, TOLERANCE);
		original.setPolygon(polygon, METER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 2000.0, TOLERANCE);
		original.setPolygon(polygon, CENTIMETER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 20.0, TOLERANCE);
		original.setPolygon(polygon, MILLIMETER);

		Location originalLoc(ObjectPositionData::getTemplate(), ObjectPositionData::getBaseTag());
		originalLoc = original;

		ObjectPositionData parsed = extract<ObjectPositionData>(originalLoc);
		ObjectPositionData::Hypothesis parsedHyp = parsed.getPositionHypotheses()[0];

		BOOST_REQUIRE_EQUAL(parsedHyp.getGenerator(), originalHyp.getGenerator());
		BOOST_REQUIRE_EQUAL(parsedHyp.getTimestamp(), originalHyp.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsedHyp.getReliability(), originalHyp.getReliability(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsedHyp.getClassLabel(), originalHyp.getClassLabel());
		BOOST_REQUIRE_EQUAL(parsed.getReference(), original.getReference());
		BOOST_REQUIRE_EQUAL(parsed.getCoordinateKind(), original.getCoordinateKind());
		BOOST_REQUIRE_EQUAL(original.getPolygon(MILLIMETER).size(), parsed.getPolygon(MILLIMETER).size());
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[0].first, parsed.getPolygon(MILLIMETER)[0].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[1].first, parsed.getPolygon(MILLIMETER)[1].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, parsed.getPolygon(MILLIMETER)[2].first, TOLERANCE);

		BOOST_REQUIRE(original == parsed);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("ObjectPositionData.xml"), ObjectPositionData::getBaseTag());
	ObjectPositionData parsed = extract<ObjectPositionData>(fileContents);
	ObjectPositionData::Hypothesis parsedHyp = parsed.getPositionHypotheses()[0];

	BOOST_REQUIRE_EQUAL("BoostDetector", parsedHyp.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsedHyp.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsedHyp.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(0.6, parsedHyp.getReliability(), TOLERANCE);
	BOOST_REQUIRE_EQUAL("smacks", parsedHyp.getClassLabel());
	BOOST_REQUIRE_EQUAL("world", parsed.getReference());
	BOOST_REQUIRE_EQUAL("absolute", parsed.getCoordinateKind());
	BOOST_REQUIRE_EQUAL(3.0, parsed.getPolygon(MILLIMETER).size());
	BOOST_REQUIRE_CLOSE(0.0, parsed.getPolygon(MILLIMETER)[0].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.0, parsed.getPolygon(MILLIMETER)[1].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0, parsed.getPolygon(MILLIMETER)[2].first, TOLERANCE);
}

BOOST_AUTO_TEST_SUITE_END()
