/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * MovementDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>
#include <cmath>

#include "btl/data/person/PersonData.h"
#include "btl/units/AngleUnit.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PersonDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		PersonData original("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setFacing(true);
		original.setHasNeighbour(true);
		original.setPoi(false);
		original.setTalking(true);
		original.setWalking(false);
		original.setAngle(0.23,RADIAN);
		original.setDistance(-12.45,METER);
		original.setGaze(213.56);
		original.setNeighbourDistance(42.123,METER);
		original.setIdentId(42);
		original.setNeighbourId(7);
		original.setIdentName("foobar");

		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.234,RADIAN);
		face->setDistance(13.0345,METER);
		face->setHeight(-234.4,METER);
		original.setFace(face);

		boost::shared_ptr<Legs> legs = boost::shared_ptr<Legs>(new Legs);
		legs->setGenerator("legsensor");
		legs->setAngle(-23.45,DEGREE);
		legs->setDistance(2134.34,METER);
		legs->setAngleVariance(0.45,DEGREE);
		legs->setDistanceVariance(1.34,METER);
		legs->setNumInvolved(2134);
		legs->setInvolved1(4543);
		legs->setInvolved2(1223);
		legs->setPair(true);
		original.setLegs(legs);

		Location originalLoc(PersonData::getTemplate(), PersonData::getBaseTag());
		originalLoc = original;

		PersonData parsed = extract<PersonData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.isFacing(), original.isFacing());
		BOOST_REQUIRE_EQUAL(parsed.hasNeighbour(), original.hasNeighbour());
		BOOST_REQUIRE_EQUAL(parsed.isPoi(), original.isPoi());
		BOOST_REQUIRE_EQUAL(parsed.isTalking(), original.isTalking());
		BOOST_REQUIRE_EQUAL(parsed.isWalking(), original.isWalking());
		BOOST_REQUIRE_CLOSE(parsed.getAngle(RADIAN), original.getAngle(RADIAN), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getDistance(METER), original.getDistance(METER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getGaze(), original.getGaze(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getNeighbourDistance(METER), original.getNeighbourDistance(METER), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getIdentId(), original.getIdentId());
		BOOST_REQUIRE_EQUAL(parsed.getNeighbourId(), original.getNeighbourId());
		BOOST_REQUIRE_EQUAL(parsed.getIdentName(), original.getIdentName());
		BOOST_REQUIRE_EQUAL(*(parsed.getFace()), *(original.getFace()));
		BOOST_REQUIRE_EQUAL(*(parsed.getLegs()), *(original.getLegs()));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	try {

		Location fileContents(readFile("Person.xml"), PersonData::getBaseTag());
		PersonData parsed = extract<PersonData>(fileContents);

		BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
		BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());

		BOOST_REQUIRE_CLOSE((float) 2.1467, parsed.getAngle(RADIAN), 0.01);
		BOOST_REQUIRE_CLOSE((float) 1235, parsed.getDistance(MILLIMETER), 0.001);
		BOOST_REQUIRE_EQUAL(false, parsed.isFacing());
		BOOST_REQUIRE_CLOSE((float) (M_PI / 2.0), parsed.getGaze(), 0.01);
		BOOST_REQUIRE_EQUAL(123, parsed.getIdentId());
		BOOST_REQUIRE_EQUAL("peter", parsed.getIdentName());
		BOOST_REQUIRE_CLOSE((float) 456, parsed
				.getNeighbourDistance(MILLIMETER), 0.0001);
		BOOST_REQUIRE_EQUAL(12, parsed.getNeighbourId());
		BOOST_REQUIRE_EQUAL(true, parsed.isPoi());
		BOOST_REQUIRE_EQUAL(true, parsed.isTalking());
		BOOST_REQUIRE_EQUAL(true, parsed.isWalking());

		Face face;
		face.setTimestamp(Timestamp(12345, 67890));
		face.setGenerator("facesensor");
		face.setAngle(0.45,RADIAN);
		face.setDistance(77.23,MILLIMETER);
		face.setHeight(-12.34,MILLIMETER);
		face.setAngleVariance(0.34,RADIAN);
		face.setHeightVariance(12.12,MILLIMETER);
		face.setDistanceVariance(-3.45,MILLIMETER);
		face.setLineOfSight(30);
		face.setFaceWidthAngle(13.56,RADIAN);
		face.setReliability(0.67);
		face.setImagePositionX(123);
		face.setImagePositionY(92);
		face.setImageWidth(200);
		face.setImageHeight(42);
		BOOST_REQUIRE(parsed.hasFace());
		BOOST_REQUIRE_EQUAL(face, *(parsed.getFace()));

		Legs legs;
		legs.setTimestamp(Timestamp(12345, 67890));
		legs.setGenerator("legsensor");
		legs.setAngle(0.45,RADIAN);
		legs.setDistance(77.23,MILLIMETER);
		legs.setAngleVariance(0.45,RADIAN);
		legs.setDistanceVariance(1.34,MILLIMETER);
		legs.setNumInvolved(2134);
		legs.setInvolved1(4543);
		legs.setInvolved2(1223);
		legs.setPair(true);
		BOOST_REQUIRE(parsed.hasLegs());
		BOOST_REQUIRE_EQUAL(legs, *(parsed.getLegs()));
		BOOST_REQUIRE_EQUAL(legs, *(parsed.getLegs()));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testVariablePercepts) {

	try {

		PersonData original("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setFacing(true);
		original.setHasNeighbour(true);
		original.setPoi(false);
		original.setTalking(true);
		original.setWalking(false);
		original.setAngle(0.23,RADIAN);
		original.setDistance(-12.45,MILLIMETER);
		original.setGaze(213.56);
		original.setNeighbourDistance(42.123,MILLIMETER);
		original.setIdentId(42);
		original.setNeighbourId(7);
		original.setIdentName("foobar");

		BOOST_REQUIRE(!original.hasFace());
		BOOST_REQUIRE(!original.hasLegs());

		Location originalLoc(PersonData::getTemplate(), PersonData::getBaseTag());
		originalLoc = original;

		PersonData parsed = extract<PersonData>(originalLoc);
		BOOST_REQUIRE(!parsed.hasFace());
		BOOST_REQUIRE(!parsed.hasLegs());

		boost::shared_ptr<Face> face = boost::shared_ptr<Face>(new Face);
		face->setGenerator("facesensor");
		face->setAngle(0.234,RADIAN);
		face->setDistance(13.0345,MILLIMETER);
		face->setHeight(-234.4,MILLIMETER);
		original.setFace(face);

		original.setFace(face);
		BOOST_REQUIRE(original.hasFace());

		Location originalLoc2(PersonData::getTemplate(), PersonData::getBaseTag());
		originalLoc2 = original;

		parsed = extract<PersonData>(originalLoc2);
		BOOST_REQUIRE(parsed.hasFace());
		BOOST_REQUIRE_EQUAL(*face, *(parsed.getFace()));
		BOOST_REQUIRE(!parsed.hasLegs());

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception (") + typeid(e).name() + "): " + e.what());
	}

}

BOOST_AUTO_TEST_SUITE_END()
